@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -20,7 +20,10 @@
 #include "builtin/intl/ScopedICUObject.h"
 #include "ds/Sort.h"
 #include "gc/FreeOp.h"
+#include "js/CharacterEncoding.h"
+#include "js/PropertySpec.h"
 #include "js/RootingAPI.h"
+#include "js/StableStringChars.h"
 #include "js/TypeDecls.h"
 #include "vm/JSContext.h"
 #include "vm/SelfHosting.h"
@@ -33,13 +36,16 @@ using namespace js;
 using mozilla::AssertedCast;
 using mozilla::IsFinite;
 using mozilla::IsNaN;
-using mozilla::IsNegativeZero;
+using mozilla::IsNegative;
+using mozilla::SpecificNaN;
 
 using js::intl::CallICU;
 using js::intl::DateTimeFormatOptions;
 using js::intl::GetAvailableLocales;
 using js::intl::IcuLocale;
 
+using JS::AutoStableStringChars;
+
 const ClassOps NumberFormatObject::classOps_ = {nullptr, /* addProperty */
                                                 nullptr, /* delProperty */
                                                 nullptr, /* enumerate */
@@ -72,7 +78,7 @@ static const JSFunctionSpec numberFormat_methods[] = {
     JS_FN(js_toSource_str, numberFormat_toSource, 0, 0), JS_FS_END};
 
 static const JSPropertySpec numberFormat_properties[] = {
-    JS_SELF_HOSTED_GET("format", "Intl_NumberFormat_format_get", 0),
+    JS_SELF_HOSTED_GET("format", "$Intl_NumberFormat_format_get", 0),
     JS_STRING_SYM_PS(toStringTag, "Object", JSPROP_READONLY), JS_PS_END};
 
 /**
@@ -85,16 +91,22 @@ static bool NumberFormat(JSContext* cx, const CallArgs& args, bool construct) {
 
   // Step 2 (Inlined 9.1.14, OrdinaryCreateFromConstructor).
   RootedObject proto(cx);
-  if (!GetPrototypeFromBuiltinConstructor(cx, args, &proto)) return false;
+  if (!GetPrototypeFromBuiltinConstructor(cx, args, JSProto_Null, &proto)) {
+    return false;
+  }
 
   if (!proto) {
     proto = GlobalObject::getOrCreateNumberFormatPrototype(cx, cx->global());
-    if (!proto) return false;
+    if (!proto) {
+      return false;
+    }
   }
 
   Rooted<NumberFormatObject*> numberFormat(cx);
   numberFormat = NewObjectWithGivenProto<NumberFormatObject>(cx, proto);
-  if (!numberFormat) return false;
+  if (!numberFormat) {
+    return false;
+  }
 
   numberFormat->setReservedSlot(NumberFormatObject::INTERNALS_SLOT,
                                 NullValue());
@@ -128,12 +140,13 @@ bool js::intl_NumberFormat(JSContext* cx, unsigned argc, Value* vp) {
 }
 
 void js::NumberFormatObject::finalize(FreeOp* fop, JSObject* obj) {
-  MOZ_ASSERT(fop->onActiveCooperatingThread());
+  MOZ_ASSERT(fop->onMainThread());
 
   const Value& slot = obj->as<NumberFormatObject>().getReservedSlot(
       NumberFormatObject::UNUMBER_FORMAT_SLOT);
-  if (UNumberFormat* nf = static_cast<UNumberFormat*>(slot.toPrivate()))
+  if (UNumberFormat* nf = static_cast<UNumberFormat*>(slot.toPrivate())) {
     unum_close(nf);
+  }
 }
 
 JSObject* js::CreateNumberFormatPrototype(JSContext* cx, HandleObject Intl,
@@ -142,28 +155,40 @@ JSObject* js::CreateNumberFormatPrototype(JSContext* cx, HandleObject Intl,
   RootedFunction ctor(cx);
   ctor = GlobalObject::createConstructor(cx, &NumberFormat,
                                          cx->names().NumberFormat, 0);
-  if (!ctor) return nullptr;
+  if (!ctor) {
+    return nullptr;
+  }
 
   RootedObject proto(
       cx, GlobalObject::createBlankPrototype<PlainObject>(cx, global));
-  if (!proto) return nullptr;
+  if (!proto) {
+    return nullptr;
+  }
 
-  if (!LinkConstructorAndPrototype(cx, ctor, proto)) return nullptr;
+  if (!LinkConstructorAndPrototype(cx, ctor, proto)) {
+    return nullptr;
+  }
 
   // 11.3.2
-  if (!JS_DefineFunctions(cx, ctor, numberFormat_static_methods))
+  if (!JS_DefineFunctions(cx, ctor, numberFormat_static_methods)) {
     return nullptr;
+  }
 
   // 11.4.4
-  if (!JS_DefineFunctions(cx, proto, numberFormat_methods)) return nullptr;
+  if (!JS_DefineFunctions(cx, proto, numberFormat_methods)) {
+    return nullptr;
+  }
 
   // 11.4.2 and 11.4.3
-  if (!JS_DefineProperties(cx, proto, numberFormat_properties)) return nullptr;
+  if (!JS_DefineProperties(cx, proto, numberFormat_properties)) {
+    return nullptr;
+  }
 
   // 8.1
   RootedValue ctorValue(cx, ObjectValue(*ctor));
-  if (!DefineDataProperty(cx, Intl, cx->names().NumberFormat, ctorValue, 0))
+  if (!DefineDataProperty(cx, Intl, cx->names().NumberFormat, ctorValue, 0)) {
     return nullptr;
+  }
 
   constructor.set(ctor);
   return proto;
@@ -175,8 +200,10 @@ bool js::intl_NumberFormat_availableLocales(JSContext* cx, unsigned argc,
   MOZ_ASSERT(args.length() == 0);
 
   RootedValue result(cx);
-  if (!GetAvailableLocales(cx, unum_countAvailable, unum_getAvailable, &result))
+  if (!GetAvailableLocales(cx, unum_countAvailable, unum_getAvailable,
+                           &result)) {
     return false;
+  }
   args.rval().set(result);
   return true;
 }
@@ -186,11 +213,13 @@ bool js::intl_numberingSystem(JSContext* cx, unsigned argc, Value* vp) {
   MOZ_ASSERT(args.length() == 1);
   MOZ_ASSERT(args[0].isString());
 
-  JSAutoByteString locale(cx, args[0].toString());
-  if (!locale) return false;
+  UniqueChars locale = intl::EncodeLocale(cx, args[0].toString());
+  if (!locale) {
+    return false;
+  }
 
   UErrorCode status = U_ZERO_ERROR;
-  UNumberingSystem* numbers = unumsys_open(IcuLocale(locale.ptr()), &status);
+  UNumberingSystem* numbers = unumsys_open(IcuLocale(locale.get()), &status);
   if (U_FAILURE(status)) {
     intl::ReportInternalError(cx);
     return false;
@@ -199,8 +228,15 @@ bool js::intl_numberingSystem(JSContext* cx, unsigned argc, Value* vp) {
   ScopedICUObject<UNumberingSystem, unumsys_close> toClose(numbers);
 
   const char* name = unumsys_getName(numbers);
-  JSString* jsname = JS_NewStringCopyZ(cx, name);
-  if (!jsname) return false;
+  if (!name) {
+    intl::ReportInternalError(cx);
+    return false;
+  }
+
+  JSString* jsname = NewStringCopyZ<CanGC>(cx, name);
+  if (!jsname) {
+    return false;
+  }
 
   args.rval().setString(jsname);
   return true;
@@ -215,12 +251,17 @@ static UNumberFormat* NewUNumberFormat(
   RootedValue value(cx);
 
   RootedObject internals(cx, intl::GetInternalsObject(cx, numberFormat));
-  if (!internals) return nullptr;
-
-  if (!GetProperty(cx, internals, internals, cx->names().locale, &value))
+  if (!internals) {
     return nullptr;
-  JSAutoByteString locale(cx, value.toString());
-  if (!locale) return nullptr;
+  }
+
+  if (!GetProperty(cx, internals, internals, cx->names().locale, &value)) {
+    return nullptr;
+  }
+  UniqueChars locale = intl::EncodeLocale(cx, value.toString());
+  if (!locale) {
+    return nullptr;
+  }
 
   // UNumberFormat options with default values
   UNumberFormatStyle uStyle = UNUM_DECIMAL;
@@ -239,28 +280,38 @@ static UNumberFormat* NewUNumberFormat(
   // We don't need to look at numberingSystem - it can only be set via
   // the Unicode locale extension and is therefore already set on locale.
 
-  if (!GetProperty(cx, internals, internals, cx->names().style, &value))
+  if (!GetProperty(cx, internals, internals, cx->names().style, &value)) {
     return nullptr;
+  }
 
   {
     JSLinearString* style = value.toString()->ensureLinear(cx);
-    if (!style) return nullptr;
+    if (!style) {
+      return nullptr;
+    }
 
     if (StringEqualsAscii(style, "currency")) {
-      if (!GetProperty(cx, internals, internals, cx->names().currency, &value))
+      if (!GetProperty(cx, internals, internals, cx->names().currency,
+                       &value)) {
         return nullptr;
+      }
       currency = value.toString();
       MOZ_ASSERT(currency->length() == 3,
                  "IsWellFormedCurrencyCode permits only length-3 strings");
-      if (!stableChars.initTwoByte(cx, currency)) return nullptr;
+      if (!stableChars.initTwoByte(cx, currency)) {
+        return nullptr;
+      }
       // uCurrency remains owned by stableChars.
       uCurrency = stableChars.twoByteRange().begin().get();
 
       if (!GetProperty(cx, internals, internals, cx->names().currencyDisplay,
-                       &value))
+                       &value)) {
         return nullptr;
+      }
       JSLinearString* currencyDisplay = value.toString()->ensureLinear(cx);
-      if (!currencyDisplay) return nullptr;
+      if (!currencyDisplay) {
+        return nullptr;
+      }
       if (StringEqualsAscii(currencyDisplay, "code")) {
         uStyle = UNUM_CURRENCY_ISO;
       } else if (StringEqualsAscii(currencyDisplay, "symbol")) {
@@ -278,43 +329,51 @@ static UNumberFormat* NewUNumberFormat(
   }
 
   bool hasP;
-  if (!HasProperty(cx, internals, cx->names().minimumSignificantDigits, &hasP))
+  if (!HasProperty(cx, internals, cx->names().minimumSignificantDigits,
+                   &hasP)) {
     return nullptr;
+  }
 
   if (hasP) {
     if (!GetProperty(cx, internals, internals,
-                     cx->names().minimumSignificantDigits, &value))
+                     cx->names().minimumSignificantDigits, &value)) {
       return nullptr;
+    }
     uMinimumSignificantDigits = value.toInt32();
 
     if (!GetProperty(cx, internals, internals,
-                     cx->names().maximumSignificantDigits, &value))
+                     cx->names().maximumSignificantDigits, &value)) {
       return nullptr;
+    }
     uMaximumSignificantDigits = value.toInt32();
   } else {
     if (!GetProperty(cx, internals, internals, cx->names().minimumIntegerDigits,
-                     &value))
+                     &value)) {
       return nullptr;
+    }
     uMinimumIntegerDigits = AssertedCast<uint32_t>(value.toInt32());
 
     if (!GetProperty(cx, internals, internals,
-                     cx->names().minimumFractionDigits, &value))
+                     cx->names().minimumFractionDigits, &value)) {
       return nullptr;
+    }
     uMinimumFractionDigits = AssertedCast<uint32_t>(value.toInt32());
 
     if (!GetProperty(cx, internals, internals,
-                     cx->names().maximumFractionDigits, &value))
+                     cx->names().maximumFractionDigits, &value)) {
       return nullptr;
+    }
     uMaximumFractionDigits = AssertedCast<uint32_t>(value.toInt32());
   }
 
-  if (!GetProperty(cx, internals, internals, cx->names().useGrouping, &value))
+  if (!GetProperty(cx, internals, internals, cx->names().useGrouping, &value)) {
     return nullptr;
+  }
   uUseGrouping = value.toBoolean();
 
   UErrorCode status = U_ZERO_ERROR;
   UNumberFormat* nf =
-      unum_open(uStyle, nullptr, 0, IcuLocale(locale.ptr()), nullptr, &status);
+      unum_open(uStyle, nullptr, 0, IcuLocale(locale.get()), nullptr, &status);
   if (U_FAILURE(status)) {
     intl::ReportInternalError(cx);
     return nullptr;
@@ -348,12 +407,16 @@ static UNumberFormat* NewUNumberFormat(
 static JSString* PartitionNumberPattern(JSContext* cx, UNumberFormat* nf,
                                         double* x,
                                         UFieldPositionIterator* fpositer) {
-  // PartitionNumberPattern doesn't consider -0.0 to be negative.
-  if (IsNegativeZero(*x)) *x = 0.0;
+  // ICU incorrectly formats NaN values with the sign bit set, as if they
+  // were negative.  Replace all NaNs with a single pattern with sign bit
+  // unset ("positive", that is) until ICU is fixed.
+  if (MOZ_UNLIKELY(IsNaN(*x))) {
+    *x = SpecificNaN<double>(0, 1);
+  }
 
-  return CallICU(cx, [nf, x, fpositer](UChar* chars, int32_t size,
-                                       UErrorCode* status) {
-    return unum_formatDoubleForFields(nf, *x, chars, size, fpositer, status);
+  return CallICU(cx, [nf, d = *x, fpositer](UChar* chars, int32_t size,
+                                            UErrorCode* status) {
+    return unum_formatDoubleForFields(nf, d, chars, size, fpositer, status);
   });
 }
 
@@ -362,7 +425,9 @@ static bool intl_FormatNumber(JSContext* cx, UNumberFormat* nf, double x,
   // Passing null for |fpositer| will just not compute partition information,
   // letting us common up all ICU number-formatting code.
   JSString* str = PartitionNumberPattern(cx, nf, &x, nullptr);
-  if (!str) return false;
+  if (!str) {
+    return false;
+  }
 
   result.setString(str);
   return true;
@@ -379,8 +444,12 @@ static FieldType GetFieldTypeForNumberField(UNumberFormatFields fieldName,
   // version-testing #ifdefs, should cross-version divergence occur.
   switch (fieldName) {
     case UNUM_INTEGER_FIELD:
-      if (IsNaN(d)) return &JSAtomState::nan;
-      if (!IsFinite(d)) return &JSAtomState::infinity;
+      if (IsNaN(d)) {
+        return &JSAtomState::nan;
+      }
+      if (!IsFinite(d)) {
+        return &JSAtomState::infinity;
+      }
       return &JSAtomState::integer;
 
     case UNUM_GROUPING_SEPARATOR_FIELD:
@@ -393,13 +462,15 @@ static FieldType GetFieldTypeForNumberField(UNumberFormatFields fieldName,
       return &JSAtomState::fraction;
 
     case UNUM_SIGN_FIELD: {
-      MOZ_ASSERT(!IsNegativeZero(d),
-                 "-0 should have been excluded by PartitionNumberPattern");
-
       // Manual trawling through the ICU call graph appears to indicate that
       // the basic formatting we request will never include a positive sign.
       // But this analysis may be mistaken, so don't absolutely trust it.
-      return d < 0 ? &JSAtomState::minusSign : &JSAtomState::plusSign;
+      MOZ_ASSERT(!IsNaN(d),
+                 "ICU appearing not to produce positive-sign among fields, "
+                 "plus our coercing all NaNs to one with sign bit unset "
+                 "(i.e. \"positive\"), means we shouldn't reach here with a "
+                 "NaN value");
+      return IsNegative(d) ? &JSAtomState::minusSign : &JSAtomState::plusSign;
     }
 
     case UNUM_PERCENT_FIELD:
@@ -424,6 +495,24 @@ static FieldType GetFieldTypeForNumberField(UNumberFormatFields fieldName,
           "and scientific notation were never requested");
       break;
 
+#ifndef U_HIDE_DRAFT_API
+#  if U_ICU_VERSION_MAJOR_NUM >= 64
+    case UNUM_MEASURE_UNIT_FIELD:
+      MOZ_ASSERT_UNREACHABLE(
+          "unexpected measure unit field found, even though "
+          "we don't use any user-defined patterns that "
+          "would require a measure unit field");
+      break;
+
+    case UNUM_COMPACT_FIELD:
+      MOZ_ASSERT_UNREACHABLE(
+          "unexpected compact field found, even though "
+          "we don't use any user-defined patterns that "
+          "would require a compact number notation");
+      break;
+#  endif
+#endif
+
 #ifndef U_HIDE_DEPRECATED_API
     case UNUM_FIELD_COUNT:
       MOZ_ASSERT_UNREACHABLE(
@@ -454,10 +543,14 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
       fpositer);
 
   RootedString overallResult(cx, PartitionNumberPattern(cx, nf, &x, fpositer));
-  if (!overallResult) return false;
+  if (!overallResult) {
+    return false;
+  }
 
   RootedArrayObject partsArray(cx, NewDenseEmptyArray(cx));
-  if (!partsArray) return false;
+  if (!partsArray) {
+    return false;
+  }
 
   // First, vacuum up fields in the overall formatted string.
 
@@ -487,14 +580,17 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
     FieldType type =
         GetFieldTypeForNumberField(UNumberFormatFields(fieldInt), x);
     if (!fields.emplaceBack(uint32_t(beginIndexInt), uint32_t(endIndexInt),
-                            type))
+                            type)) {
       return false;
+    }
   }
 
   // Second, merge sort the fields vector.  Expand the vector to have scratch
   // space for performing the sort.
   size_t fieldsLen = fields.length();
-  if (!fields.resizeUninitialized(fieldsLen * 2)) return false;
+  if (!fields.resizeUninitialized(fieldsLen * 2)) {
+    return false;
+  }
 
   MOZ_ALWAYS_TRUE(MergeSort(
       fields.begin(), fieldsLen, fields.begin() + fieldsLen,
@@ -507,7 +603,9 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
       }));
 
   // Deallocate the scratch space.
-  if (!fields.resize(fieldsLen)) return false;
+  if (!fields.resize(fieldsLen)) {
+    return false;
+  }
 
   // Third, iterate over the sorted field list to generate a sequence of
   // parts (what ECMA-402 actually exposes).  A part is a maximal character
@@ -585,8 +683,9 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
                     fields[enclosingFields.back()].end >= end);
 
       while (enclosingFields.length() > 0 &&
-             fields[enclosingFields.back()].end == end)
+             fields[enclosingFields.back()].end == end) {
         enclosingFields.popBack();
+      }
     }
 
     bool nextPartInternal(Part* part) {
@@ -659,7 +758,9 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
         // enclosing field.  (If there are no nested fields, don't
         // bother pushing a field that'd be immediately popped.)
         if (current->end > next->begin) {
-          if (!enclosingFields.append(index - 1)) return false;
+          if (!enclosingFields.append(index - 1)) {
+            return false;
+          }
         }
 
         // Do so until the next field begins after this one.
@@ -697,7 +798,9 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
         return true;
       }
 
-      if (!nextPartInternal(part)) return false;
+      if (!nextPartInternal(part)) {
+        return false;
+      }
 
       *hasPart = true;
       lastEnd = part->end;
@@ -715,9 +818,13 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
   do {
     bool hasPart;
     Part part;
-    if (!gen.nextPart(&hasPart, &part)) return false;
+    if (!gen.nextPart(&hasPart, &part)) {
+      return false;
+    }
 
-    if (!hasPart) break;
+    if (!hasPart) {
+      break;
+    }
 
     FieldType type = part.type;
     size_t endIndex = part.end;
@@ -725,22 +832,30 @@ static bool intl_FormatNumberToParts(JSContext* cx, UNumberFormat* nf, double x,
     MOZ_ASSERT(lastEndIndex < endIndex);
 
     singlePart = NewBuiltinClassInstance<PlainObject>(cx);
-    if (!singlePart) return false;
+    if (!singlePart) {
+      return false;
+    }
 
     propVal.setString(cx->names().*type);
-    if (!DefineDataProperty(cx, singlePart, cx->names().type, propVal))
+    if (!DefineDataProperty(cx, singlePart, cx->names().type, propVal)) {
       return false;
+    }
 
     JSLinearString* partSubstr = NewDependentString(
         cx, overallResult, lastEndIndex, endIndex - lastEndIndex);
-    if (!partSubstr) return false;
+    if (!partSubstr) {
+      return false;
+    }
 
     propVal.setString(partSubstr);
-    if (!DefineDataProperty(cx, singlePart, cx->names().value, propVal))
+    if (!DefineDataProperty(cx, singlePart, cx->names().value, propVal)) {
       return false;
+    }
 
     propVal.setObject(*singlePart);
-    if (!DefineDataElement(cx, partsArray, partIndex, propVal)) return false;
+    if (!DefineDataElement(cx, partsArray, partIndex, propVal)) {
+      return false;
+    }
 
     lastEndIndex = endIndex;
     partIndex++;
@@ -770,14 +885,17 @@ bool js::intl_FormatNumber(JSContext* cx, unsigned argc, Value* vp) {
   UNumberFormat* nf = static_cast<UNumberFormat*>(priv);
   if (!nf) {
     nf = NewUNumberFormat(cx, numberFormat);
-    if (!nf) return false;
+    if (!nf) {
+      return false;
+    }
     numberFormat->setReservedSlot(NumberFormatObject::UNUMBER_FORMAT_SLOT,
                                   PrivateValue(nf));
   }
 
   // Use the UNumberFormat to actually format the number.
-  if (args[2].toBoolean())
+  if (args[2].toBoolean()) {
     return intl_FormatNumberToParts(cx, nf, args[1].toNumber(), args.rval());
+  }
 
   return intl_FormatNumber(cx, nf, args[1].toNumber(), args.rval());
 }
