@@ -9,7 +9,7 @@ assertEq(wasmEvalText(`
         br_table $out $out
       end
     end
-    get_local $p
+    local.get $p
     br_if 0
   )
   (export "f" $f)
