@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "js/RegExp.h"
+#include "js/RegExpFlags.h"
 #include "jsapi-tests/tests.h"
 
 BEGIN_TEST(testObjectIsRegExp) {
@@ -11,12 +13,12 @@ BEGIN_TEST(testObjectIsRegExp) {
 
   EVAL("new Object", &val);
   JS::RootedObject obj(cx, val.toObjectOrNull());
-  CHECK(JS_ObjectIsRegExp(cx, obj, &isRegExp));
+  CHECK(JS::ObjectIsRegExp(cx, obj, &isRegExp));
   CHECK(!isRegExp);
 
   EVAL("/foopy/", &val);
   obj = val.toObjectOrNull();
-  CHECK(JS_ObjectIsRegExp(cx, obj, &isRegExp));
+  CHECK(JS::ObjectIsRegExp(cx, obj, &isRegExp));
   CHECK(isRegExp);
 
   return true;
@@ -29,15 +31,19 @@ BEGIN_TEST(testGetRegExpFlags) {
 
   EVAL("/foopy/", &val);
   obj = val.toObjectOrNull();
-  CHECK_EQUAL(JS_GetRegExpFlags(cx, obj), 0u);
+  CHECK_EQUAL(JS::GetRegExpFlags(cx, obj),
+              JS::RegExpFlags(JS::RegExpFlag::NoFlags));
 
   EVAL("/foopy/g", &val);
   obj = val.toObjectOrNull();
-  CHECK_EQUAL(JS_GetRegExpFlags(cx, obj), JSREG_GLOB);
+  CHECK_EQUAL(JS::GetRegExpFlags(cx, obj),
+              JS::RegExpFlags(JS::RegExpFlag::Global));
 
   EVAL("/foopy/gi", &val);
   obj = val.toObjectOrNull();
-  CHECK_EQUAL(JS_GetRegExpFlags(cx, obj), (JSREG_FOLD | JSREG_GLOB));
+  CHECK_EQUAL(
+      JS::GetRegExpFlags(cx, obj),
+      JS::RegExpFlags(JS::RegExpFlag::Global | JS::RegExpFlag::IgnoreCase));
 
   return true;
 }
@@ -49,7 +55,8 @@ BEGIN_TEST(testGetRegExpSource) {
 
   EVAL("/foopy/", &val);
   obj = val.toObjectOrNull();
-  JSString* source = JS_GetRegExpSource(cx, obj);
+  JSString* source = JS::GetRegExpSource(cx, obj);
+  CHECK(source);
   CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(source), "foopy"));
 
   return true;
