@@ -1,4 +1,4 @@
-// |reftest| skip -- BigInt is not supported
+// |reftest| skip-if(!this.hasOwnProperty('BigInt')) -- BigInt is not enabled unconditionally
 // Copyright (C) 2017 Caio Lima. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -6,10 +6,17 @@
 description: Octal prefixed String should be parsed to BigInt according StringToBigInt
 esid: sec-string-to-bigint
 info: |
-  Apply the algorithm in 3.1.3.1 with the following changes:
+  ToBigInt ( argument )
 
-  - Replace the StrUnsignedDecimalLiteral production with DecimalDigits
-    to not allow decimal points or exponents.
+  String:
+
+  Let n be StringToBigInt(prim).
+  If n is NaN, throw a SyntaxError exception.
+  Return n.
+
+  StringToBigInt ( argument )
+
+  Replace the StrUnsignedDecimalLiteral production with DecimalDigits to not allow Infinity, decimal points, or exponents.
 
 features: [BigInt]
 ---*/
@@ -22,5 +29,4 @@ assert.sameValue(BigInt("0O7"), 7n);
 assert.sameValue(BigInt("0O10"), 8n);
 assert.sameValue(BigInt("0O20"), 16n);
 
-
 reportCompare(0, 0);
