@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -16,6 +16,11 @@ inline JSString* js::ErrorObject::fileName(JSContext* cx) const {
   return slot.isString() ? slot.toString() : cx->names().empty;
 }
 
+inline uint32_t js::ErrorObject::sourceId() const {
+  const HeapSlot& slot = getReservedSlotRef(SOURCEID_SLOT);
+  return slot.isInt32() ? slot.toInt32() : 0;
+}
+
 inline uint32_t js::ErrorObject::lineNumber() const {
   const HeapSlot& slot = getReservedSlotRef(LINENUMBER_SLOT);
   return slot.isInt32() ? slot.toInt32() : 0;
@@ -30,4 +35,9 @@ inline JSObject* js::ErrorObject::stack() const {
   return getReservedSlotRef(STACK_SLOT).toObjectOrNull();
 }
 
+inline uint64_t js::ErrorObject::timeWarpTarget() const {
+  const HeapSlot& slot = getReservedSlotRef(TIME_WARP_SLOT);
+  return slot.isDouble() ? slot.toDouble() : 0;
+}
+
 #endif /* vm_ErrorObject_inl_h */
