@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,25 +11,26 @@
 
 #include "ds/IdValuePair.h"
 #include "gc/Barrier.h"
+#include "gc/GCTrace.h"
 #include "js/CharacterEncoding.h"
 #include "js/GCHashTable.h"
 #include "js/TypeDecls.h"
 #include "vm/TaggedProto.h"
-#include "vm/TypeInference.h"
+#include "vm/TypeSet.h"
 
 namespace js {
 
 class TypeDescr;
-class UnboxedLayout;
 
 class PreliminaryObjectArrayWithTemplate;
 class TypeNewScript;
-class HeapTypeSet;
 class AutoClearTypeInferenceStateOnOOM;
+class AutoSweepObjectGroup;
 class CompilerConstraintList;
+class ObjectGroupRealm;
 
 namespace gc {
-void MergeCompartments(JSCompartment* source, JSCompartment* target);
+void MergeRealms(JS::Realm* source, JS::Realm* target);
 }  // namespace gc
 
 /*
@@ -62,7 +63,7 @@ enum NewObjectKind {
 };
 
 /*
- * Lazy object groups overview.
+ * [SMDOC] Type-Inference lazy ObjectGroup
  *
  * Object groups which represent at most one JS object are constructed lazily.
  * These include groups for native functions, standard classes, scripted
@@ -84,33 +85,102 @@ enum NewObjectKind {
 
 /* Type information about an object accessed by a script. */
 class ObjectGroup : public gc::TenuredCell {
-  friend class gc::GCRuntime;
+ public:
+  class Property;
 
+ private:
   /* Class shared by objects in this group. */
-  const Class* clasp_;
+  const Class* clasp_;  // set by constructor
 
   /* Prototype shared by objects in this group. */
-  GCPtr<TaggedProto> proto_;
+  GCPtr<TaggedProto> proto_;  // set by constructor
 
-  /* Compartment shared by objects in this group. */
-  JSCompartment* compartment_;
+  /* Realm shared by objects in this group. */
+  JS::Realm* realm_;
+  ;  // set by constructor
+
+  /* Flags for this group. */
+  ObjectGroupFlags flags_;  // set by constructor
+
+  // If non-null, holds additional information about this object, whose
+  // format is indicated by the object's addendum kind.
+  void* addendum_ = nullptr;
+
+  /*
+   * [SMDOC] Type-Inference object properties
+   *
+   * Properties of this object.
+   *
+   * The type sets in the properties of a group describe the possible values
+   * that can be read out of that property in actual JS objects. In native
+   * objects, property types account for plain data properties (those with a
+   * slot and no getter or setter hook) and dense elements. In typed objects,
+   * property types account for object and value properties and elements in the
+   * object.
+   *
+   * For accesses on these properties, the correspondence is as follows:
+   *
+   * 1. If the group has unknownProperties(), the possible properties and
+   *    value types for associated JSObjects are unknown.
+   *
+   * 2. Otherwise, for any |obj| in |group|, and any |id| which is a property
+   *    in |obj|, before obj->getProperty(id) the property in |group| for
+   *    |id| must reflect the result of the getProperty.
+   *
+   * There are several exceptions to this:
+   *
+   * 1. For properties of global JS objects which are undefined at the point
+   *    where the property was (lazily) generated, the property type set will
+   *    remain empty, and the 'undefined' type will only be added after a
+   *    subsequent assignment or deletion. After these properties have been
+   *    assigned a defined value, the only way they can become undefined
+   *    again is after such an assign or deletion.
+   *
+   * 2. Array lengths are special cased by the compiler and VM and are not
+   *    reflected in property types.
+   *
+   * 3. In typed objects, the initial values of properties (null pointers and
+   *    undefined values) are not reflected in the property types. These values
+   *    are always possible when reading the property.
+   *
+   * We establish these by using write barriers on calls to setProperty and
+   * defineProperty which are on native properties, and on any jitcode which
+   * might update the property with a new type.
+   */
+  Property** propertySet = nullptr;
+
+  // END OF PROPERTIES
+
+ private:
+  static inline uint32_t offsetOfClasp() {
+    return offsetof(ObjectGroup, clasp_);
+  }
+
+  static inline uint32_t offsetOfProto() {
+    return offsetof(ObjectGroup, proto_);
+  }
+
+  static inline uint32_t offsetOfRealm() {
+    return offsetof(ObjectGroup, realm_);
+  }
+
+  static inline uint32_t offsetOfFlags() {
+    return offsetof(ObjectGroup, flags_);
+  }
+
+  static inline uint32_t offsetOfAddendum() {
+    return offsetof(ObjectGroup, addendum_);
+  }
+
+  friend class gc::GCRuntime;
+  friend class gc::GCTrace;
+
+  // See JSObject::offsetOfGroup() comment.
+  friend class js::jit::MacroAssembler;
 
  public:
   const Class* clasp() const { return clasp_; }
 
-  void setClasp(const Class* clasp) {
-    MOZ_ASSERT(JS::StringIsASCII(clasp->name));
-    MOZ_ASSERT(hasUncacheableClass());
-    clasp_ = clasp;
-  }
-
-  // Certain optimizations may mutate the class of an ObjectGroup - and thus
-  // all objects in it - after it is created. If true, the JIT must not
-  // assume objects of a previously seen group have the same class as before.
-  //
-  // See: TryConvertToUnboxedLayout
-  bool hasUncacheableClass() const { return clasp_->isNative(); }
-
   bool hasDynamicPrototype() const { return proto_.isDynamic(); }
 
   const GCPtr<TaggedProto>& proto() const { return proto_; }
@@ -138,12 +208,11 @@ class ObjectGroup : public gc::TenuredCell {
     return res;
   }
 
-  JSCompartment* compartment() const { return compartment_; }
-  JSCompartment* maybeCompartment() const { return compartment(); }
-
- private:
-  /* Flags for this group. */
-  ObjectGroupFlags flags_;
+  JS::Compartment* compartment() const {
+    return JS::GetCompartmentForRealm(realm_);
+  }
+  JS::Compartment* maybeCompartment() const { return compartment(); }
+  JS::Realm* realm() const { return realm_; }
 
  public:
   // Kinds of addendums which can be attached to ObjectGroups.
@@ -162,25 +231,11 @@ class ObjectGroup : public gc::TenuredCell {
     // PreliminaryObjectArrayWithTemplate.
     Addendum_PreliminaryObjects,
 
-    // When objects in this group have an unboxed representation, the
-    // addendum stores an UnboxedLayout (which might have a TypeNewScript
-    // as well, if the group is also constructed using 'new').
-    Addendum_UnboxedLayout,
-
-    // If this group is used by objects that have been converted from an
-    // unboxed representation and/or have the same allocation kind as such
-    // objects, the addendum points to that unboxed group.
-    Addendum_OriginalUnboxedGroup,
-
     // When used by typed objects, the addendum stores a TypeDescr.
     Addendum_TypeDescr
   };
 
  private:
-  // If non-null, holds additional information about this object, whose
-  // format is indicated by the object's addendum kind.
-  void* addendum_;
-
   void setAddendum(AddendumKind kind, void* addendum, bool writeBarrier = true);
 
   AddendumKind addendumKind() const {
@@ -189,21 +244,21 @@ class ObjectGroup : public gc::TenuredCell {
   }
 
   TypeNewScript* newScriptDontCheckGeneration() const {
-    if (addendumKind() == Addendum_NewScript)
+    if (addendumKind() == Addendum_NewScript) {
       return reinterpret_cast<TypeNewScript*>(addendum_);
+    }
     return nullptr;
   }
 
-  TypeNewScript* anyNewScript();
   void detachNewScript(bool writeBarrier, ObjectGroup* replacement);
 
   ObjectGroupFlags flagsDontCheckGeneration() const { return flags_; }
 
  public:
-  inline ObjectGroupFlags flags();
-  inline void addFlags(ObjectGroupFlags flags);
-  inline void clearFlags(ObjectGroupFlags flags);
-  inline TypeNewScript* newScript();
+  inline ObjectGroupFlags flags(const AutoSweepObjectGroup&);
+  inline void addFlags(const AutoSweepObjectGroup&, ObjectGroupFlags flags);
+  inline void clearFlags(const AutoSweepObjectGroup&, ObjectGroupFlags flags);
+  inline TypeNewScript* newScript(const AutoSweepObjectGroup& sweep);
 
   void setNewScript(TypeNewScript* newScript) {
     MOZ_ASSERT(newScript);
@@ -211,12 +266,14 @@ class ObjectGroup : public gc::TenuredCell {
   }
   void detachNewScript() { setAddendum(Addendum_None, nullptr); }
 
-  inline PreliminaryObjectArrayWithTemplate* maybePreliminaryObjects();
+  inline PreliminaryObjectArrayWithTemplate* maybePreliminaryObjects(
+      const AutoSweepObjectGroup& sweep);
 
   PreliminaryObjectArrayWithTemplate*
   maybePreliminaryObjectsDontCheckGeneration() {
-    if (addendumKind() == Addendum_PreliminaryObjects)
+    if (addendumKind() == Addendum_PreliminaryObjects) {
       return reinterpret_cast<PreliminaryObjectArrayWithTemplate*>(addendum_);
+    }
     return nullptr;
   }
 
@@ -230,51 +287,20 @@ class ObjectGroup : public gc::TenuredCell {
     setAddendum(Addendum_None, nullptr);
   }
 
-  bool hasUnanalyzedPreliminaryObjects() {
-    return (newScriptDontCheckGeneration() &&
-            !newScriptDontCheckGeneration()->analyzed()) ||
-           maybePreliminaryObjectsDontCheckGeneration();
-  }
-
-  inline UnboxedLayout* maybeUnboxedLayout();
-  inline UnboxedLayout& unboxedLayout();
-
-  UnboxedLayout* maybeUnboxedLayoutDontCheckGeneration() const {
-    if (addendumKind() == Addendum_UnboxedLayout)
-      return reinterpret_cast<UnboxedLayout*>(addendum_);
-    return nullptr;
-  }
-
-  UnboxedLayout& unboxedLayoutDontCheckGeneration() const {
-    MOZ_ASSERT(addendumKind() == Addendum_UnboxedLayout);
-    return *maybeUnboxedLayoutDontCheckGeneration();
-  }
-
-  void setUnboxedLayout(UnboxedLayout* layout) {
-    setAddendum(Addendum_UnboxedLayout, layout);
-  }
-
-  ObjectGroup* maybeOriginalUnboxedGroup() const {
-    if (addendumKind() == Addendum_OriginalUnboxedGroup)
-      return reinterpret_cast<ObjectGroup*>(addendum_);
-    return nullptr;
-  }
-
-  void setOriginalUnboxedGroup(ObjectGroup* group) {
-    setAddendum(Addendum_OriginalUnboxedGroup, group);
-  }
+  inline bool hasUnanalyzedPreliminaryObjects();
 
   TypeDescr* maybeTypeDescr() {
     // Note: there is no need to sweep when accessing the type descriptor
     // of an object, as it is strongly held and immutable.
-    if (addendumKind() == Addendum_TypeDescr)
-      return reinterpret_cast<TypeDescr*>(addendum_);
+    if (addendumKind() == Addendum_TypeDescr) {
+      return &typeDescr();
+    }
     return nullptr;
   }
 
   TypeDescr& typeDescr() {
     MOZ_ASSERT(addendumKind() == Addendum_TypeDescr);
-    return *maybeTypeDescr();
+    return *reinterpret_cast<TypeDescr*>(addendum_);
   }
 
   void setTypeDescr(TypeDescr* descr) {
@@ -284,8 +310,9 @@ class ObjectGroup : public gc::TenuredCell {
   JSFunction* maybeInterpretedFunction() {
     // Note: as with type descriptors, there is no need to sweep when
     // accessing the interpreted function associated with an object.
-    if (addendumKind() == Addendum_InterpretedFunction)
+    if (addendumKind() == Addendum_InterpretedFunction) {
       return reinterpret_cast<JSFunction*>(addendum_);
+    }
     return nullptr;
   }
 
@@ -311,63 +338,25 @@ class ObjectGroup : public gc::TenuredCell {
     static jsid getKey(Property* p) { return p->id; }
   };
 
- private:
-  /*
-   * Properties of this object.
-   *
-   * The type sets in the properties of a group describe the possible values
-   * that can be read out of that property in actual JS objects. In native
-   * objects, property types account for plain data properties (those with a
-   * slot and no getter or setter hook) and dense elements. In typed objects
-   * and unboxed objects, property types account for object and value
-   * properties and elements in the object, and expando properties in unboxed
-   * objects.
-   *
-   * For accesses on these properties, the correspondence is as follows:
-   *
-   * 1. If the group has unknownProperties(), the possible properties and
-   *    value types for associated JSObjects are unknown.
-   *
-   * 2. Otherwise, for any |obj| in |group|, and any |id| which is a property
-   *    in |obj|, before obj->getProperty(id) the property in |group| for
-   *    |id| must reflect the result of the getProperty.
-   *
-   * There are several exceptions to this:
-   *
-   * 1. For properties of global JS objects which are undefined at the point
-   *    where the property was (lazily) generated, the property type set will
-   *    remain empty, and the 'undefined' type will only be added after a
-   *    subsequent assignment or deletion. After these properties have been
-   *    assigned a defined value, the only way they can become undefined
-   *    again is after such an assign or deletion.
-   *
-   * 2. Array lengths are special cased by the compiler and VM and are not
-   *    reflected in property types.
-   *
-   * 3. In typed objects (but not unboxed objects), the initial values of
-   *    properties (null pointers and undefined values) are not reflected in
-   *    the property types. These values are always possible when reading the
-   *    property.
-   *
-   * We establish these by using write barriers on calls to setProperty and
-   * defineProperty which are on native properties, and on any jitcode which
-   * might update the property with a new type.
-   */
-  Property** propertySet;
-
  public:
-  inline ObjectGroup(const Class* clasp, TaggedProto proto, JSCompartment* comp,
+  inline ObjectGroup(const Class* clasp, TaggedProto proto, JS::Realm* realm,
                      ObjectGroupFlags initialFlags);
 
-  inline bool hasAnyFlags(ObjectGroupFlags flags);
-  inline bool hasAllFlags(ObjectGroupFlags flags);
+  inline bool hasAnyFlags(const AutoSweepObjectGroup& sweep,
+                          ObjectGroupFlags flags);
+  inline bool hasAllFlags(const AutoSweepObjectGroup& sweep,
+                          ObjectGroupFlags flags);
 
+  bool hasAnyFlagsDontCheckGeneration(ObjectGroupFlags flags) {
+    MOZ_ASSERT((flags & OBJECT_FLAG_DYNAMIC_MASK) == flags);
+    return !!(this->flagsDontCheckGeneration() & flags);
+  }
   bool hasAllFlagsDontCheckGeneration(ObjectGroupFlags flags) {
     MOZ_ASSERT((flags & OBJECT_FLAG_DYNAMIC_MASK) == flags);
     return (this->flagsDontCheckGeneration() & flags) == flags;
   }
 
-  inline bool unknownProperties();
+  inline bool unknownProperties(const AutoSweepObjectGroup& sweep);
 
   bool unknownPropertiesDontCheckGeneration() {
     MOZ_ASSERT_IF(flagsDontCheckGeneration() & OBJECT_FLAG_UNKNOWN_PROPERTIES,
@@ -375,22 +364,26 @@ class ObjectGroup : public gc::TenuredCell {
     return !!(flagsDontCheckGeneration() & OBJECT_FLAG_UNKNOWN_PROPERTIES);
   }
 
-  inline bool shouldPreTenure();
+  inline bool shouldPreTenure(const AutoSweepObjectGroup& sweep);
+  inline bool shouldPreTenureDontCheckGeneration();
 
   gc::InitialHeap initialHeap(CompilerConstraintList* constraints);
 
-  inline bool canPreTenure();
-  inline bool fromAllocationSite();
-  inline void setShouldPreTenure(JSContext* cx);
+  inline bool canPreTenure(const AutoSweepObjectGroup& sweep);
+  inline bool fromAllocationSite(const AutoSweepObjectGroup& sweep);
+  inline void setShouldPreTenure(const AutoSweepObjectGroup& sweep,
+                                 JSContext* cx);
 
   /*
    * Get or create a property of this object. Only call this for properties
    * which a script accesses explicitly.
    */
-  inline HeapTypeSet* getProperty(JSContext* cx, JSObject* obj, jsid id);
+  inline HeapTypeSet* getProperty(const AutoSweepObjectGroup& sweep,
+                                  JSContext* cx, JSObject* obj, jsid id);
 
   /* Get a property only if it already exists. */
-  MOZ_ALWAYS_INLINE HeapTypeSet* maybeGetProperty(jsid id);
+  MOZ_ALWAYS_INLINE HeapTypeSet* maybeGetProperty(
+      const AutoSweepObjectGroup& sweep, jsid id);
   MOZ_ALWAYS_INLINE HeapTypeSet* maybeGetPropertyDontCheckGeneration(jsid id);
 
   /*
@@ -398,34 +391,32 @@ class ObjectGroup : public gc::TenuredCell {
    * in the hash case (see SET_ARRAY_SIZE in TypeInference-inl.h), and
    * getProperty may return nullptr.
    */
-  inline unsigned getPropertyCount();
-  inline Property* getProperty(unsigned i);
+  inline unsigned getPropertyCount(const AutoSweepObjectGroup& sweep);
+  inline Property* getProperty(const AutoSweepObjectGroup& sweep, unsigned i);
 
   /* Helpers */
 
-  void updateNewPropertyTypes(JSContext* cx, JSObject* obj, jsid id,
-                              HeapTypeSet* types);
+  void updateNewPropertyTypes(const AutoSweepObjectGroup& sweep, JSContext* cx,
+                              JSObject* obj, jsid id, HeapTypeSet* types);
   void addDefiniteProperties(JSContext* cx, Shape* shape);
-  bool matchDefiniteProperties(HandleObject obj);
   void markPropertyNonData(JSContext* cx, JSObject* obj, jsid id);
   void markPropertyNonWritable(JSContext* cx, JSObject* obj, jsid id);
-  void markStateChange(JSContext* cx);
-  void setFlags(JSContext* cx, ObjectGroupFlags flags);
-  void markUnknown(JSContext* cx);
+  void markStateChange(const AutoSweepObjectGroup& sweep, JSContext* cx);
+  void setFlags(const AutoSweepObjectGroup& sweep, JSContext* cx,
+                ObjectGroupFlags flags);
+  void markUnknown(const AutoSweepObjectGroup& sweep, JSContext* cx);
   void maybeClearNewScriptOnOOM();
   void clearNewScript(JSContext* cx, ObjectGroup* replacement = nullptr);
 
-  void print();
+  void print(const AutoSweepObjectGroup& sweep);
 
-  inline void clearProperties();
+  inline void clearProperties(const AutoSweepObjectGroup& sweep);
   void traceChildren(JSTracer* trc);
 
   inline bool needsSweep();
-  inline void maybeSweep(AutoClearTypeInferenceStateOnOOM* oom);
+  void sweep(const AutoSweepObjectGroup& sweep);
 
  private:
-  void sweep(AutoClearTypeInferenceStateOnOOM* oom);
-
   uint32_t generation() {
     return (flags_ & OBJECT_FLAG_GENERATION_MASK) >>
            OBJECT_FLAG_GENERATION_SHIFT;
@@ -445,44 +436,15 @@ class ObjectGroup : public gc::TenuredCell {
 
   static const JS::TraceKind TraceKind = JS::TraceKind::ObjectGroup;
 
- private:
-  // See JSObject::offsetOfGroup() comment.
-  friend class js::jit::MacroAssembler;
-
-  static inline uint32_t offsetOfClasp() {
-    return offsetof(ObjectGroup, clasp_);
-  }
-
-  static inline uint32_t offsetOfProto() {
-    return offsetof(ObjectGroup, proto_);
-  }
-
-  static inline uint32_t offsetOfCompartment() {
-    return offsetof(ObjectGroup, compartment_);
-  }
-
-  static inline uint32_t offsetOfAddendum() {
-    return offsetof(ObjectGroup, addendum_);
-  }
-
-  static inline uint32_t offsetOfFlags() {
-    return offsetof(ObjectGroup, flags_);
-  }
-
  public:
   const ObjectGroupFlags* addressOfFlags() const { return &flags_; }
 
-  // Get the bit pattern stored in an object's addendum when it has an
-  // original unboxed group.
-  static inline int32_t addendumOriginalUnboxedGroupValue() {
-    return Addendum_OriginalUnboxedGroup << OBJECT_FLAG_ADDENDUM_SHIFT;
-  }
-
-  inline uint32_t basePropertyCount();
+  inline uint32_t basePropertyCount(const AutoSweepObjectGroup& sweep);
   inline uint32_t basePropertyCountDontCheckGeneration();
 
  private:
-  inline void setBasePropertyCount(uint32_t count);
+  inline void setBasePropertyCount(const AutoSweepObjectGroup& sweep,
+                                   uint32_t count);
 
   static void staticAsserts() {
     JS_STATIC_ASSERT(offsetof(ObjectGroup, proto_) ==
@@ -500,18 +462,17 @@ class ObjectGroup : public gc::TenuredCell {
   // Whether to make a singleton object at an allocation site.
   static bool useSingletonForAllocationSite(JSScript* script, jsbytecode* pc,
                                             JSProtoKey key);
-  static bool useSingletonForAllocationSite(JSScript* script, jsbytecode* pc,
-                                            const Class* clasp);
 
-  // Static accessors for ObjectGroupCompartment NewTable.
+  // Static accessors for ObjectGroupRealm NewTable.
 
   static ObjectGroup* defaultNewGroup(JSContext* cx, const Class* clasp,
                                       TaggedProto proto,
                                       JSObject* associated = nullptr);
-  static ObjectGroup* lazySingletonGroup(JSContext* cx, const Class* clasp,
-                                         TaggedProto proto);
+  static ObjectGroup* lazySingletonGroup(JSContext* cx, ObjectGroup* oldGroup,
+                                         const Class* clasp, TaggedProto proto);
 
-  static void setDefaultNewGroupUnknown(JSContext* cx, const js::Class* clasp,
+  static void setDefaultNewGroupUnknown(JSContext* cx, ObjectGroupRealm& realm,
+                                        const js::Class* clasp,
                                         JS::HandleObject obj);
 
 #ifdef DEBUG
@@ -519,7 +480,7 @@ class ObjectGroup : public gc::TenuredCell {
                                  ObjectGroup* group);
 #endif
 
-  // Static accessors for ObjectGroupCompartment ArrayObjectTable and
+  // Static accessors for ObjectGroupRealm ArrayObjectTable and
   // PlainObjectTable.
 
   enum class NewArrayKind {
@@ -534,12 +495,12 @@ class ObjectGroup : public gc::TenuredCell {
       JSContext* cx, const Value* vp, size_t length, NewObjectKind newKind,
       NewArrayKind arrayKind = NewArrayKind::Normal);
 
-  // Create a PlainObject or UnboxedPlainObject with the specified properties
-  // and a group specialized for those properties.
+  // Create a PlainObject with the specified properties and a group specialized
+  // for those properties.
   static JSObject* newPlainObject(JSContext* cx, IdValuePair* properties,
                                   size_t nproperties, NewObjectKind newKind);
 
-  // Static accessors for ObjectGroupCompartment AllocationSiteTable.
+  // Static accessors for ObjectGroupRealm AllocationSiteTable.
 
   // Get a non-singleton group to use for objects created at the specified
   // allocation site.
@@ -563,45 +524,21 @@ class ObjectGroup : public gc::TenuredCell {
   static ArrayObject* getCopyOnWriteObject(JSScript* script, jsbytecode* pc);
 
   // Returns false if not found.
-  static bool findAllocationSite(JSContext* cx, ObjectGroup* group,
+  static bool findAllocationSite(JSContext* cx, const ObjectGroup* group,
                                  JSScript** script, uint32_t* offset);
 
  private:
   static ObjectGroup* defaultNewGroup(JSContext* cx, JSProtoKey key);
 };
 
-// Structure used to manage the groups in a compartment.
-class ObjectGroupCompartment {
-  friend class ObjectGroup;
-
+// Structure used to manage the groups in a realm.
+class ObjectGroupRealm {
+ private:
   class NewTable;
 
-  // Set of default 'new' or lazy groups in the compartment.
-  NewTable* defaultNewTable;
-  NewTable* lazyTable;
-
-  // Cache for defaultNewGroup. Purged on GC.
-  class DefaultNewGroupCache {
-    ObjectGroup* group_;
-    JSObject* associated_;
-
-   public:
-    DefaultNewGroupCache() { purge(); }
-
-    void purge() { group_ = nullptr; }
-    void put(ObjectGroup* group, JSObject* associated) {
-      group_ = group;
-      associated_ = associated;
-    }
-
-    MOZ_ALWAYS_INLINE ObjectGroup* lookup(const Class* clasp, TaggedProto proto,
-                                          JSObject* associated);
-  };
-  DefaultNewGroupCache defaultNewGroupCache;
-
   struct ArrayObjectKey;
   using ArrayObjectTable =
-      js::GCRekeyableHashMap<ArrayObjectKey, ReadBarrieredObjectGroup,
+      js::GCRekeyableHashMap<ArrayObjectKey, WeakHeapPtrObjectGroup,
                              ArrayObjectKey, SystemAllocPolicy>;
 
   struct PlainObjectKey;
@@ -613,6 +550,31 @@ class ObjectGroupCompartment {
       JS::GCHashMap<PlainObjectKey, PlainObjectEntry, PlainObjectKey,
                     SystemAllocPolicy, PlainObjectTableSweepPolicy>;
 
+  class AllocationSiteTable;
+
+ private:
+  // Set of default 'new' or lazy groups in the realm.
+  NewTable* defaultNewTable = nullptr;
+  NewTable* lazyTable = nullptr;
+
+  // This cache is purged on GC.
+  class DefaultNewGroupCache {
+    ObjectGroup* group_;
+    JSObject* associated_;
+
+   public:
+    DefaultNewGroupCache() : associated_(nullptr) { purge(); }
+
+    void purge() { group_ = nullptr; }
+    void put(ObjectGroup* group, JSObject* associated) {
+      group_ = group;
+      associated_ = associated;
+    }
+
+    MOZ_ALWAYS_INLINE ObjectGroup* lookup(const Class* clasp, TaggedProto proto,
+                                          JSObject* associated);
+  } defaultNewGroupCache = {};
+
   // Tables for managing groups common to the contents of large script
   // singleton objects and JSON objects. These are vanilla ArrayObjects and
   // PlainObjects, so we distinguish the groups of different ones by looking
@@ -622,28 +584,39 @@ class ObjectGroupCompartment {
   // and of the same element type will share a group. All singleton/JSON
   // objects which have the same shape and property types will also share a
   // group. We don't try to collate arrays or objects with type mismatches.
-  ArrayObjectTable* arrayObjectTable;
-  PlainObjectTable* plainObjectTable;
-
-  struct AllocationSiteKey;
-  class AllocationSiteTable;
+  ArrayObjectTable* arrayObjectTable = nullptr;
+  PlainObjectTable* plainObjectTable = nullptr;
 
   // Table for referencing types of objects keyed to an allocation site.
-  AllocationSiteTable* allocationSiteTable;
+  AllocationSiteTable* allocationSiteTable = nullptr;
 
-  // A single per-compartment ObjectGroup for all calls to StringSplitString.
+  // A single per-realm ObjectGroup for all calls to StringSplitString.
   // StringSplitString is always called from self-hosted code, and conceptually
   // the return object for a string.split(string) operation should have a
   // unified type.  Having a global group for this also allows us to remove
   // the hash-table lookup that would be required if we allocated this group
   // on the basis of call-site pc.
-  ReadBarrieredObjectGroup stringSplitStringGroup;
+  WeakHeapPtrObjectGroup stringSplitStringGroup = {};
+
+  // END OF PROPERTIES
+
+ private:
+  friend class ObjectGroup;
+
+  struct AllocationSiteKey;
+  friend struct MovableCellHasher<AllocationSiteKey>;
 
  public:
   struct NewEntry;
 
-  ObjectGroupCompartment();
-  ~ObjectGroupCompartment();
+  ObjectGroupRealm() = default;
+  ~ObjectGroupRealm();
+
+  ObjectGroupRealm(ObjectGroupRealm&) = delete;
+  void operator=(ObjectGroupRealm&) = delete;
+
+  static ObjectGroupRealm& get(const ObjectGroup* group);
+  static ObjectGroupRealm& getForNewObject(JSContext* cx);
 
   void replaceAllocationSiteGroup(JSScript* script, jsbytecode* pc,
                                   JSProtoKey kind, ObjectGroup* group);
@@ -653,8 +626,8 @@ class ObjectGroupCompartment {
   void replaceDefaultNewGroup(const Class* clasp, TaggedProto proto,
                               JSObject* associated, ObjectGroup* group);
 
-  static ObjectGroup* makeGroup(JSContext* cx, const Class* clasp,
-                                Handle<TaggedProto> proto,
+  static ObjectGroup* makeGroup(JSContext* cx, JS::Realm* realm,
+                                const Class* clasp, Handle<TaggedProto> proto,
                                 ObjectGroupFlags initialFlags = 0);
 
   static ObjectGroup* getStringSplitStringGroup(JSContext* cx);
@@ -663,7 +636,7 @@ class ObjectGroupCompartment {
                               size_t* allocationSiteTables,
                               size_t* arrayGroupTables,
                               size_t* plainObjectGroupTables,
-                              size_t* compartmentTables);
+                              size_t* realmTables);
 
   void clearTables();
 
