@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -23,6 +23,7 @@ class SavedFrame : public NativeObject {
 
  public:
   static const Class class_;
+  static const Class protoClass_;
   static const JSPropertySpec protoAccessors[];
   static const JSFunctionSpec protoFunctions[];
   static const JSFunctionSpec staticFunctions[];
@@ -30,6 +31,7 @@ class SavedFrame : public NativeObject {
   // Prototype methods and properties to be exposed to JS.
   static bool construct(JSContext* cx, unsigned argc, Value* vp);
   static bool sourceProperty(JSContext* cx, unsigned argc, Value* vp);
+  static bool sourceIdProperty(JSContext* cx, unsigned argc, Value* vp);
   static bool lineProperty(JSContext* cx, unsigned argc, Value* vp);
   static bool columnProperty(JSContext* cx, unsigned argc, Value* vp);
   static bool functionDisplayNameProperty(JSContext* cx, unsigned argc,
@@ -43,6 +45,7 @@ class SavedFrame : public NativeObject {
 
   // Convenient getters for SavedFrame's reserved slots for use from C++.
   JSAtom* getSource();
+  uint32_t getSourceId();
   uint32_t getLine();
   uint32_t getColumn();
   JSAtom* getFunctionDisplayName();
@@ -50,6 +53,11 @@ class SavedFrame : public NativeObject {
   SavedFrame* getParent() const;
   JSPrincipals* getPrincipals();
   bool isSelfHosted(JSContext* cx);
+  bool isWasm();
+
+  // When isWasm():
+  uint32_t wasmFuncIndex();
+  uint32_t wasmBytecodeOffset();
 
   // Iterator for use with C++11 range based for loops, eg:
   //
@@ -95,45 +103,21 @@ class SavedFrame : public NativeObject {
     RootedIterator end() { return RootedIterator(); }
   };
 
-  static bool isSavedFrameAndNotProto(JSObject& obj) {
-    return obj.is<SavedFrame>() &&
-           !obj.as<SavedFrame>().getReservedSlot(JSSLOT_SOURCE).isNull();
-  }
-
-  static bool isSavedFrameOrWrapperAndNotProto(JSObject& obj) {
-    auto unwrapped = CheckedUnwrap(&obj);
-    if (!unwrapped) return false;
-    return isSavedFrameAndNotProto(*unwrapped);
-  }
-
   struct Lookup;
   struct HashPolicy;
 
-  typedef JS::GCHashSet<ReadBarriered<SavedFrame*>, HashPolicy,
+  typedef JS::GCHashSet<WeakHeapPtr<SavedFrame*>, HashPolicy,
                         SystemAllocPolicy>
       Set;
 
-  class AutoLookupVector;
-
-  class MOZ_STACK_CLASS HandleLookup {
-    friend class AutoLookupVector;
-
-    Lookup& lookup;
-
-    explicit HandleLookup(Lookup& lookup) : lookup(lookup) {}
-
-   public:
-    inline Lookup& get() { return lookup; }
-    inline Lookup* operator->() { return &lookup; }
-  };
-
  private:
   static SavedFrame* create(JSContext* cx);
   static MOZ_MUST_USE bool finishSavedFrameInit(JSContext* cx,
                                                 HandleObject ctor,
                                                 HandleObject proto);
-  void initFromLookup(JSContext* cx, HandleLookup lookup);
+  void initFromLookup(JSContext* cx, Handle<Lookup> lookup);
   void initSource(JSAtom* source);
+  void initSourceId(uint32_t id);
   void initLine(uint32_t line);
   void initColumn(uint32_t column);
   void initFunctionDisplayName(JSAtom* maybeName);
@@ -145,6 +129,7 @@ class SavedFrame : public NativeObject {
   enum {
     // The reserved slots in the SavedFrame class.
     JSSLOT_SOURCE,
+    JSSLOT_SOURCEID,
     JSSLOT_LINE,
     JSSLOT_COLUMN,
     JSSLOT_FUNCTIONDISPLAYNAME,
@@ -167,22 +152,30 @@ struct SavedFrame::HashPolicy {
   static HashNumber hash(const Lookup& lookup);
   static bool match(SavedFrame* existing, const Lookup& lookup);
 
-  typedef ReadBarriered<SavedFrame*> Key;
+  typedef WeakHeapPtr<SavedFrame*> Key;
   static void rekey(Key& key, const Key& newKey);
 };
 
+}  // namespace js
+
+namespace mozilla {
+
 template <>
-struct FallibleHashMethods<SavedFrame::HashPolicy> {
+struct FallibleHashMethods<js::SavedFrame::HashPolicy> {
   template <typename Lookup>
   static bool hasHash(Lookup&& l) {
-    return SavedFrame::HashPolicy::hasHash(mozilla::Forward<Lookup>(l));
+    return js::SavedFrame::HashPolicy::hasHash(std::forward<Lookup>(l));
   }
   template <typename Lookup>
   static bool ensureHash(Lookup&& l) {
-    return SavedFrame::HashPolicy::ensureHash(mozilla::Forward<Lookup>(l));
+    return js::SavedFrame::HashPolicy::ensureHash(std::forward<Lookup>(l));
   }
 };
 
+}  // namespace mozilla
+
+namespace js {
+
 // Assert that if the given object is not null, that it must be either a
 // SavedFrame object or wrapper (Xray or CCW) around a SavedFrame object.
 inline void AssertObjectIsSavedFrameOrWrapper(JSContext* cx,
@@ -257,6 +250,8 @@ class ConcreteStackFrame<SavedFrame> : public BaseStackFrame {
     return AtomOrTwoByteChars(source);
   }
 
+  uint32_t sourceId() const override { return get().getSourceId(); }
+
   AtomOrTwoByteChars functionDisplayName() const override {
     auto name = get().getFunctionDisplayName();
     return AtomOrTwoByteChars(name);
@@ -266,7 +261,9 @@ class ConcreteStackFrame<SavedFrame> : public BaseStackFrame {
     JSObject* prev = &get();
     JSObject* next = prev;
     js::TraceRoot(trc, &next, "ConcreteStackFrame<SavedFrame>::ptr");
-    if (next != prev) ptr = next;
+    if (next != prev) {
+      ptr = next;
+    }
   }
 
   bool isSelfHosted(JSContext* cx) const override {
