@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: set ts=8 sts=2 et sw=2 tw=80:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,20 +9,24 @@
 
 #include "mozilla/GuardObjects.h"
 #include "mozilla/LinkedList.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/Vector.h"
+
+#include <utility>
+
+#include "jsapi.h"
 
 #include "js/AllocPolicy.h"
 #include "js/HashTable.h"
+#include "js/TraceLoggerAPI.h"
 #include "js/TypeDecls.h"
 #include "js/Vector.h"
+#include "threading/LockGuard.h"
 #include "vm/MutexIDs.h"
 #include "vm/TraceLoggingGraph.h"
 #include "vm/TraceLoggingTypes.h"
 
-namespace JS {
-class ReadOnlyCompileOptions;
-}  // namespace JS
-
 namespace js {
 
 namespace jit {
@@ -30,7 +34,7 @@ class CompileRuntime;
 }  // namespace jit
 
 /*
- * Tracelogging overview.
+ * [SMDOC] Tracelogging overview.
  *
  * Tracelogging makes it possible to trace the occurrence of a single event
  * and/or the start and stop of an event. This is implemented with as low
@@ -104,6 +108,7 @@ class TraceLoggerEvent {
       return (TraceLoggerEventPayload*)payload_;
     }
     void setEventPayload(TraceLoggerEventPayload* payload) {
+      MOZ_ASSERT(payload);
       payload_ = (uintptr_t)payload;
       MOZ_ASSERT((payload_ & 1) == 0);
     }
@@ -125,8 +130,8 @@ class TraceLoggerEvent {
   TraceLoggerEvent() : payload_() {}
   explicit TraceLoggerEvent(TraceLoggerTextId textId);
   TraceLoggerEvent(TraceLoggerTextId type, JSScript* script);
-  TraceLoggerEvent(TraceLoggerTextId type, const char* filename, size_t line,
-                   size_t column);
+  TraceLoggerEvent(TraceLoggerTextId type, const char* filename, uint32_t line,
+                   uint32_t column);
   explicit TraceLoggerEvent(const char* text);
   TraceLoggerEvent(const TraceLoggerEvent& event);
   TraceLoggerEvent& operator=(const TraceLoggerEvent& other);
@@ -147,8 +152,8 @@ class TraceLoggerEvent {
   TraceLoggerEvent() {}
   explicit TraceLoggerEvent(TraceLoggerTextId textId) {}
   TraceLoggerEvent(TraceLoggerTextId type, JSScript* script) {}
-  TraceLoggerEvent(TraceLoggerTextId type, const char* filename, size_t line,
-                   size_t column) {}
+  TraceLoggerEvent(TraceLoggerTextId type, const char* filename, uint32_t line,
+                   uint32_t column) {}
   explicit TraceLoggerEvent(const char* text) {}
   TraceLoggerEvent(const TraceLoggerEvent& event) {}
   TraceLoggerEvent& operator=(const TraceLoggerEvent& other) { return *this; };
@@ -164,29 +169,39 @@ bool CurrentThreadOwnsTraceLoggerThreadStateLock();
 
 /**
  * An internal class holding the string information to report, together with an
- * unique id, a useCount and a pointerCount. Whenever this useCount reaches 0,
- * this event cannot get started/stopped anymore. Consumers may still request
- * the string information through maybeEventText below, but this may not
- * succeed: when the use count becomes zero, a payload may be deleted by any
- * thread holding the TraceLoggerThreadState lock, after that the pointers have
- * been cleared out of the pointerMap. That means pointerCount needs to be zero.
+ * unique id, and an useCount. Whenever this useCount reaches 0, this event
+ * cannot get started/stopped anymore. Consumers may still request the
+ * string information through maybeEventText below, but this may not succeed:
+ * when the use count becomes zero, a payload may be deleted by any thread
+ * holding the TraceLoggerThreadState lock, after that the pointers have been
+ * cleared out of the dictionaryVector.
  */
 class TraceLoggerEventPayload {
   uint32_t textId_;
-  UniqueChars string_;
+  uint32_t dictionaryId_;
+  mozilla::Maybe<uint32_t> line_;
+  mozilla::Maybe<uint32_t> col_;
   mozilla::Atomic<uint32_t> uses_;
-  mozilla::Atomic<uint32_t> pointerCount_;
 
  public:
-  TraceLoggerEventPayload(uint32_t textId, char* string)
-      : textId_(textId), string_(string), uses_(0) {}
+  TraceLoggerEventPayload(uint32_t textId, uint32_t dictionaryId)
+      : textId_(textId),
+        dictionaryId_(dictionaryId),
+        line_(mozilla::Nothing()),
+        col_(mozilla::Nothing()),
+        uses_(0) {}
 
   ~TraceLoggerEventPayload() { MOZ_ASSERT(uses_ == 0); }
 
+  void setLine(uint32_t line) { line_ = mozilla::Some(line); }
+  void setColumn(uint32_t col) { col_ = mozilla::Some(col); }
+
+  mozilla::Maybe<uint32_t> line() { return line_; }
+  mozilla::Maybe<uint32_t> column() { return col_; }
   uint32_t textId() { return textId_; }
-  const char* string() { return string_.get(); }
+  uint32_t dictionaryId() { return dictionaryId_; }
+  void setDictionaryId(uint32_t dictId) { dictionaryId_ = dictId; }
   uint32_t uses() { return uses_; }
-  uint32_t pointerCount() { return pointerCount_; }
 
   // Payloads may have their use count change at any time, *except* the count
   // can only go from zero to non-zero while the thread state lock is held.
@@ -197,25 +212,20 @@ class TraceLoggerEventPayload {
     uses_++;
   }
   void release() { uses_--; }
-  void incPointerCount() {
-    MOZ_ASSERT(CurrentThreadOwnsTraceLoggerThreadStateLock());
-    pointerCount_++;
-  }
-  void decPointerCount() {
-    MOZ_ASSERT(CurrentThreadOwnsTraceLoggerThreadStateLock());
-    pointerCount_--;
-  }
-  size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
-    return mallocSizeOf(string_.get());
-  }
   size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
-    return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
+    return mallocSizeOf(this);
   }
 };
 
 // Per thread trace logger state.
 class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
 #ifdef JS_TRACE_LOGGING
+  friend JS::TraceLoggerIdImpl;
+  friend JS::TraceLoggerTimeStampImpl;
+  friend JS::TraceLoggerDurationImpl;
+  friend JS::TraceLoggerLineNoImpl;
+  friend JS::TraceLoggerColNoImpl;
+
  private:
   uint32_t enabled_;
   bool failed;
@@ -229,10 +239,10 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
   // event.
   uint32_t iteration_;
 
-#ifdef DEBUG
+#  ifdef DEBUG
   typedef Vector<uint32_t, 1, js::SystemAllocPolicy> GraphStack;
   GraphStack graphStack;
-#endif
+#  endif
 
  public:
   AutoTraceLog* top;
@@ -246,6 +256,8 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
   bool init(uint32_t loggerId);
   void initGraph();
 
+  void clear();
+
   bool enable();
   bool enable(JSContext* cx);
   bool disable(bool force = false, const char* = "");
@@ -284,13 +296,6 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
     *size = events.size();
   }
 
-  // Extract the details filename, lineNumber and columnNumber out of a event
-  // containing script information.
-  void extractScriptDetails(uint32_t textId, const char** filename,
-                            size_t* filename_len, const char** lineno,
-                            size_t* lineno_len, const char** colno,
-                            size_t* colno_len);
-
   bool lostEvents(uint32_t lastIteration, uint32_t lastSize) {
     // If still logging in the same iteration, there are no lost events.
     if (lastIteration == iteration_) {
@@ -301,8 +306,9 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
     // If we are in the next consecutive iteration we are only sure we
     // didn't lose any events when the lastSize equals the maximum size
     // 'events' can get.
-    if (lastIteration == iteration_ - 1 && lastSize == events.maxSize())
+    if (lastIteration == iteration_ - 1 && lastSize == events.maxSize()) {
       return false;
+    }
 
     return true;
   }
@@ -316,7 +322,6 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
     MOZ_ASSERT(text);
     return text;
   };
-  bool textIdIsScriptEvent(uint32_t id);
 
  public:
   // Log an event (no start/stop, only the timestamp is recorded).
@@ -348,62 +353,103 @@ class TraceLoggerThread : public mozilla::LinkedListElement<TraceLoggerThread> {
 // Process wide trace logger state.
 class TraceLoggerThreadState {
 #ifdef JS_TRACE_LOGGING
-#ifdef DEBUG
+  friend JS::TraceLoggerDictionaryImpl;
+#  ifdef DEBUG
   bool initialized;
-#endif
+#  endif
 
   bool enabledTextIds[TraceLogger_Last];
-  bool cooperatingThreadEnabled;
+  bool mainThreadEnabled;
   bool helperThreadEnabled;
-  bool graphSpewingEnabled;
+  bool graphEnabled;
+  bool graphFileEnabled;
   bool spewErrors;
   mozilla::LinkedList<TraceLoggerThread> threadLoggers;
 
-  typedef HashMap<const void*, TraceLoggerEventPayload*,
-                  PointerHasher<const void*>, SystemAllocPolicy>
-      PointerHashMap;
+  // Any events that carry a payload are saved in this hash map.
+  // The key is the event textId, and the value is a pointer to
+  // the payload object.
   typedef HashMap<uint32_t, TraceLoggerEventPayload*, DefaultHasher<uint32_t>,
                   SystemAllocPolicy>
-      TextIdHashMap;
-  PointerHashMap pointerMap;
-  TextIdHashMap textIdPayloads;
+      TextIdToPayloadMap;
+
+  // The dictionary vector is used to store all of the custom event strings
+  // that are referenced by the payload objects.
+  typedef mozilla::Vector<UniqueChars, 0, SystemAllocPolicy> DictionaryVector;
+
+  // All payload strings are hashed and saved as a key in the payloadDictionary
+  // hash table.  The values are indices to the dictionaryData vector where the
+  // actual string is stored.  The major benefit of having this hash map is for
+  // de-duplication of JS script filenames.
+  typedef HashMap<const char*, uint32_t, mozilla::CStringHasher,
+                  SystemAllocPolicy>
+      StringHashToDictionaryMap;
+
+  TextIdToPayloadMap textIdPayloads;
+  StringHashToDictionaryMap payloadDictionary;
+  DictionaryVector dictionaryData;
+
   uint32_t nextTextId;
+  uint32_t nextDictionaryId;
 
  public:
-  uint64_t startupTime;
+  mozilla::TimeStamp startTime;
+
+  double getTimeStampOffset(mozilla::TimeStamp time) {
+    mozilla::TimeDuration delta = time - startTime;
+    return delta.ToMicroseconds();
+  }
+
+  // Mutex to guard the data structures used to hold the payload data:
+  // textIdPayloads, payloadDictionary & dictionaryData.
   Mutex lock;
 
   TraceLoggerThreadState()
       :
-#ifdef DEBUG
+#  ifdef DEBUG
         initialized(false),
-#endif
-        cooperatingThreadEnabled(false),
+#  endif
+        mainThreadEnabled(false),
         helperThreadEnabled(false),
-        graphSpewingEnabled(false),
+        graphEnabled(false),
+        graphFileEnabled(false),
         spewErrors(false),
         nextTextId(TraceLogger_Last),
-        startupTime(0),
+        nextDictionaryId(0),
         lock(js::mutexid::TraceLoggerThreadState) {
   }
 
   bool init();
   ~TraceLoggerThreadState();
 
+  void enableDefaultLogging();
+  void enableIonLogging();
+  void enableFrontendLogging();
+
+  void clear();
+  bool remapDictionaryEntries(
+      mozilla::Vector<UniqueChars, 0, SystemAllocPolicy>* newDictionary,
+      uint32_t* newNextDictionaryId);
+
   TraceLoggerThread* forCurrentThread(JSContext* cx);
   void destroyLogger(TraceLoggerThread* logger);
 
   bool isTextIdEnabled(uint32_t textId) {
-    if (textId < TraceLogger_Last) return enabledTextIds[textId];
+    if (textId < TraceLogger_Last) {
+      return enabledTextIds[textId];
+    }
     return true;
   }
   void enableTextId(JSContext* cx, uint32_t textId);
   void disableTextId(JSContext* cx, uint32_t textId);
   void maybeSpewError(const char* text) {
-    if (spewErrors) fprintf(stderr, "%s\n", text);
+    if (spewErrors) {
+      fprintf(stderr, "%s\n", text);
+    }
   }
 
   const char* maybeEventText(uint32_t id);
+  const char* maybeEventText(TraceLoggerEventPayload* p);
 
   void purgeUnusedPayloads();
 
@@ -414,17 +460,26 @@ class TraceLoggerThreadState {
   TraceLoggerEventPayload* getOrCreateEventPayload(const char* text);
   TraceLoggerEventPayload* getOrCreateEventPayload(JSScript* script);
   TraceLoggerEventPayload* getOrCreateEventPayload(const char* filename,
-                                                   size_t lineno, size_t colno,
-                                                   const void* p);
+                                                   uint32_t lineno,
+                                                   uint32_t colno);
+  TraceLoggerEventPayload* getPayload(uint32_t id);
 
   size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf);
   size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) {
     return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
   }
+
+  bool isGraphFileEnabled() { return graphFileEnabled; }
+  bool isGraphEnabled() { return graphEnabled; }
+
+  void enableTextIdsForProfiler();
+  void disableTextIdsForProfiler();
+  void disableAllTextIds();
 #endif
 };
 
 #ifdef JS_TRACE_LOGGING
+void ResetTraceLogger();
 void DestroyTraceLoggerThreadState();
 void DestroyTraceLogger(TraceLoggerThread* logger);
 
@@ -437,26 +492,36 @@ inline TraceLoggerThread* TraceLoggerForCurrentThread(JSContext* cx = nullptr) {
 
 inline bool TraceLoggerEnable(TraceLoggerThread* logger) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) return logger->enable();
+  if (logger) {
+    return logger->enable();
+  }
 #endif
   return false;
 }
 inline bool TraceLoggerEnable(TraceLoggerThread* logger, JSContext* cx) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) return logger->enable(cx);
+  if (logger) {
+    return logger->enable(cx);
+  }
+  JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                            JSMSG_TRACELOGGER_ENABLE_FAIL, "internal error");
 #endif
   return false;
 }
 inline bool TraceLoggerDisable(TraceLoggerThread* logger) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) return logger->disable();
+  if (logger) {
+    return logger->disable();
+  }
 #endif
   return false;
 }
 inline void TraceLoggerSilentFail(TraceLoggerThread* logger,
                                   const char* error) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->silentFail(error);
+  if (logger) {
+    logger->silentFail(error);
+  }
 #endif
 }
 
@@ -472,48 +537,64 @@ inline void TraceLogDisableTextId(JSContext* cx, uint32_t textId) {}
 inline void TraceLogTimestamp(TraceLoggerThread* logger,
                               TraceLoggerTextId textId) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->logTimestamp(textId);
+  if (logger) {
+    logger->logTimestamp(textId);
+  }
 #endif
 }
 inline void TraceLogStartEvent(TraceLoggerThread* logger,
                                TraceLoggerTextId textId) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->startEvent(textId);
+  if (logger) {
+    logger->startEvent(textId);
+  }
 #endif
 }
 inline void TraceLogStartEvent(TraceLoggerThread* logger,
                                const TraceLoggerEvent& event) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->startEvent(event);
+  if (logger) {
+    logger->startEvent(event);
+  }
 #endif
 }
 inline void TraceLogStopEvent(TraceLoggerThread* logger,
                               TraceLoggerTextId textId) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->stopEvent(textId);
+  if (logger) {
+    logger->stopEvent(textId);
+  }
 #endif
 }
 inline void TraceLogStopEvent(TraceLoggerThread* logger,
                               const TraceLoggerEvent& event) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->stopEvent(event);
+  if (logger) {
+    logger->stopEvent(event);
+  }
 #endif
 }
 
 // Helper functions for assembly. May not be used otherwise.
 inline void TraceLogTimestampPrivate(TraceLoggerThread* logger, uint32_t id) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->logTimestamp(id);
+  if (logger) {
+    logger->logTimestamp(id);
+  }
 #endif
 }
 inline void TraceLogStartEventPrivate(TraceLoggerThread* logger, uint32_t id) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->startEvent(id);
+  if (logger) {
+    logger->startEvent(id);
+  }
 #endif
 }
 inline void TraceLogStopEventPrivate(TraceLoggerThread* logger, uint32_t id) {
 #ifdef JS_TRACE_LOGGING
-  if (logger) logger->stopEvent(id);
+  if (logger) {
+    logger->stopEvent(id);
+  }
 #endif
 }
 
@@ -534,7 +615,7 @@ class MOZ_RAII AutoTraceLog {
  public:
   AutoTraceLog(TraceLoggerThread* logger,
                const TraceLoggerEvent& event MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : logger(logger), isEvent(true), executed(false) {
+      : logger(logger), isEvent(true), executed(false), prev(nullptr) {
     MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     payload.event = &event;
     if (logger) {
@@ -547,7 +628,7 @@ class MOZ_RAII AutoTraceLog {
 
   AutoTraceLog(TraceLoggerThread* logger,
                TraceLoggerTextId id MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : logger(logger), isEvent(false), executed(false) {
+      : logger(logger), isEvent(false), executed(false), prev(nullptr) {
     MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     payload.id = id;
     if (logger) {
@@ -560,7 +641,9 @@ class MOZ_RAII AutoTraceLog {
 
   ~AutoTraceLog() {
     if (logger) {
-      while (this != logger->top) logger->top->stop();
+      while (this != logger->top) {
+        logger->top->stop();
+      }
       stop();
     }
   }
@@ -569,13 +652,16 @@ class MOZ_RAII AutoTraceLog {
   void stop() {
     if (!executed) {
       executed = true;
-      if (isEvent)
+      if (isEvent) {
         logger->stopEvent(*payload.event);
-      else
+      } else {
         logger->stopEvent(payload.id);
+      }
     }
 
-    if (logger->top == this) logger->top = prev;
+    if (logger->top == this) {
+      logger->top = prev;
+    }
   }
 #else
  public:
