@@ -8,6 +8,7 @@
 
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Assertions.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/StackWalk.h"
 
@@ -18,32 +19,32 @@ using namespace mozilla;
 // for _Unwind_Backtrace from libcxxrt or libunwind
 // cxxabi.h from libcxxrt implicitly includes unwind.h first
 #if defined(HAVE__UNWIND_BACKTRACE) && !defined(_GNU_SOURCE)
-#define _GNU_SOURCE
+#  define _GNU_SOURCE
 #endif
 
 #if defined(HAVE_DLOPEN) || defined(XP_DARWIN)
-#include <dlfcn.h>
+#  include <dlfcn.h>
 #endif
 
 #if (defined(XP_DARWIN) && \
      (defined(__i386) || defined(__ppc__) || defined(HAVE__UNWIND_BACKTRACE)))
-#define MOZ_STACKWALK_SUPPORTS_MACOSX 1
+#  define MOZ_STACKWALK_SUPPORTS_MACOSX 1
 #else
-#define MOZ_STACKWALK_SUPPORTS_MACOSX 0
+#  define MOZ_STACKWALK_SUPPORTS_MACOSX 0
 #endif
 
 #if (defined(linux) &&                                            \
      ((defined(__GNUC__) && (defined(__i386) || defined(PPC))) || \
       defined(HAVE__UNWIND_BACKTRACE)))
-#define MOZ_STACKWALK_SUPPORTS_LINUX 1
+#  define MOZ_STACKWALK_SUPPORTS_LINUX 1
 #else
-#define MOZ_STACKWALK_SUPPORTS_LINUX 0
+#  define MOZ_STACKWALK_SUPPORTS_LINUX 0
 #endif
 
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 1)
-#define HAVE___LIBC_STACK_END 1
+#  define HAVE___LIBC_STACK_END 1
 #else
-#define HAVE___LIBC_STACK_END 0
+#  define HAVE___LIBC_STACK_END 0
 #endif
 
 #if HAVE___LIBC_STACK_END
@@ -51,29 +52,29 @@ extern MOZ_EXPORT void* __libc_stack_end;  // from ld-linux.so
 #endif
 
 #ifdef ANDROID
-#include <algorithm>
-#include <unistd.h>
-#include <pthread.h>
+#  include <algorithm>
+#  include <unistd.h>
+#  include <pthread.h>
 #endif
 
 #if MOZ_STACKWALK_SUPPORTS_WINDOWS
 
-#include <windows.h>
-#include <process.h>
-#include <stdio.h>
-#include <malloc.h>
-#include "mozilla/ArrayUtils.h"
-#include "mozilla/Atomics.h"
-#include "mozilla/StackWalk_windows.h"
-#include "mozilla/WindowsVersion.h"
+#  include <windows.h>
+#  include <process.h>
+#  include <stdio.h>
+#  include <malloc.h>
+#  include "mozilla/ArrayUtils.h"
+#  include "mozilla/Atomics.h"
+#  include "mozilla/StackWalk_windows.h"
+#  include "mozilla/WindowsVersion.h"
 
-#include <imagehlp.h>
+#  include <imagehlp.h>
 // We need a way to know if we are building for WXP (or later), as if we are, we
 // need to use the newer 64-bit APIs. API_VERSION_NUMBER seems to fit the bill.
 // A value of 9 indicates we want to use the new APIs.
-#if API_VERSION_NUMBER < 9
-#error Too old imagehlp.h
-#endif
+#  if API_VERSION_NUMBER < 9
+#    error Too old imagehlp.h
+#  endif
 
 struct WalkStackData {
   // Are we walking the stack of the calling thread? Note that we need to avoid
@@ -94,10 +95,9 @@ struct WalkStackData {
   CONTEXT* context;
 };
 
-DWORD gStackWalkThread;
 CRITICAL_SECTION gDbgHelpCS;
 
-#ifdef _M_AMD64
+#  if defined(_M_AMD64) || defined(_M_ARM64)
 // Because various Win64 APIs acquire function-table locks, we need a way of
 // preventing stack walking while those APIs are being called. Otherwise, the
 // stack walker may suspend a thread holding such a lock, and deadlock when the
@@ -147,7 +147,7 @@ MFBT_API void UnregisterJitCodeRegion(uint8_t* aStart, size_t aSize) {
   sJitCodeRegionSize = 0;
 }
 
-#endif  // _M_AMD64
+#  endif  // _M_AMD64 || _M_ARM64
 
 // Routine to print an error message to standard error.
 static void PrintError(const char* aPrefix) {
@@ -173,58 +173,6 @@ static void InitializeDbgHelpCriticalSection() {
   initialized = true;
 }
 
-static unsigned int WINAPI WalkStackThread(void* aData);
-
-static bool EnsureWalkThreadReady() {
-  static bool walkThreadReady = false;
-  static HANDLE stackWalkThread = nullptr;
-  static HANDLE readyEvent = nullptr;
-
-  if (walkThreadReady) {
-    return walkThreadReady;
-  }
-
-  if (!stackWalkThread) {
-    readyEvent = ::CreateEvent(nullptr, FALSE /* auto-reset*/,
-                               FALSE /* initially non-signaled */, nullptr);
-    if (!readyEvent) {
-      PrintError("CreateEvent");
-      return false;
-    }
-
-    unsigned int threadID;
-    stackWalkThread = (HANDLE)_beginthreadex(nullptr, 0, WalkStackThread,
-                                             (void*)readyEvent, 0, &threadID);
-    if (!stackWalkThread) {
-      PrintError("CreateThread");
-      ::CloseHandle(readyEvent);
-      readyEvent = nullptr;
-      return false;
-    }
-    gStackWalkThread = threadID;
-    ::CloseHandle(stackWalkThread);
-  }
-
-  MOZ_ASSERT((stackWalkThread && readyEvent) ||
-             (!stackWalkThread && !readyEvent));
-
-  // The thread was created. Try to wait an arbitrary amount of time (1 second
-  // should be enough) for its event loop to start before posting events to it.
-  DWORD waitRet = ::WaitForSingleObject(readyEvent, 1000);
-  if (waitRet == WAIT_TIMEOUT) {
-    // We get a timeout if we're called during static initialization because
-    // the thread will only start executing after we return so it couldn't
-    // have signalled the event. If that is the case, give up for now and
-    // try again next time we're called.
-    return false;
-  }
-  ::CloseHandle(readyEvent);
-  stackWalkThread = nullptr;
-  readyEvent = nullptr;
-
-  return walkThreadReady = true;
-}
-
 static void WalkStackMain64(struct WalkStackData* aData) {
   // Get a context for the specified thread.
   CONTEXT context_buf;
@@ -233,36 +181,35 @@ static void WalkStackMain64(struct WalkStackData* aData) {
     context = &context_buf;
     memset(context, 0, sizeof(CONTEXT));
     context->ContextFlags = CONTEXT_FULL;
-    if (!GetThreadContext(aData->thread, context)) {
-      if (aData->walkCallingThread) {
-        PrintError("GetThreadContext");
-      }
+    if (aData->walkCallingThread) {
+      ::RtlCaptureContext(context);
+    } else if (!GetThreadContext(aData->thread, context)) {
       return;
     }
   } else {
     context = aData->context;
   }
 
-#if defined(_M_IX86) || defined(_M_IA64)
+#  if defined(_M_IX86) || defined(_M_IA64)
   // Setup initial stack frame to walk from.
   STACKFRAME64 frame64;
   memset(&frame64, 0, sizeof(frame64));
-#ifdef _M_IX86
+#    ifdef _M_IX86
   frame64.AddrPC.Offset = context->Eip;
   frame64.AddrStack.Offset = context->Esp;
   frame64.AddrFrame.Offset = context->Ebp;
-#elif defined _M_IA64
+#    elif defined _M_IA64
   frame64.AddrPC.Offset = context->StIIP;
   frame64.AddrStack.Offset = context->SP;
   frame64.AddrFrame.Offset = context->RsBSP;
-#endif
+#    endif
   frame64.AddrPC.Mode = AddrModeFlat;
   frame64.AddrStack.Mode = AddrModeFlat;
   frame64.AddrFrame.Mode = AddrModeFlat;
   frame64.AddrReturn.Mode = AddrModeFlat;
-#endif
+#  endif
 
-#ifdef _WIN64
+#  if defined(_M_AMD64) || defined(_M_ARM64)
   // If there are any active suppressions, then at least one thread (we don't
   // know which) is holding a lock that can deadlock RtlVirtualUnwind. Since
   // that thread may be the one that we're trying to unwind, we can't proceed.
@@ -276,11 +223,11 @@ static void WalkStackMain64(struct WalkStackData* aData) {
   if (sStackWalkSuppressions) {
     return;
   }
-#endif
+#  endif
 
-#ifdef _M_AMD64
+#  if defined(_M_AMD64) || defined(_M_ARM64)
   bool firstFrame = true;
-#endif
+#  endif
 
   // Skip our own stack walking frames.
   int skip = (aData->walkCallingThread ? 3 : 0) + aData->skipFrames;
@@ -290,16 +237,16 @@ static void WalkStackMain64(struct WalkStackData* aData) {
     DWORD64 addr;
     DWORD64 spaddr;
 
-#if defined(_M_IX86) || defined(_M_IA64)
+#  if defined(_M_IX86) || defined(_M_IA64)
     // 32-bit frame unwinding.
     // Debug routines are not threadsafe, so grab the lock.
     EnterCriticalSection(&gDbgHelpCS);
     BOOL ok = StackWalk64(
-#if defined _M_IA64
+#    if defined _M_IA64
         IMAGE_FILE_MACHINE_IA64,
-#elif defined _M_IX86
+#    elif defined _M_IX86
         IMAGE_FILE_MACHINE_I386,
-#endif
+#    endif
         aData->process, aData->thread, &frame64, context, nullptr,
         SymFunctionTableAccess64,  // function table access routine
         SymGetModuleBase64,        // module base routine
@@ -321,11 +268,18 @@ static void WalkStackMain64(struct WalkStackData* aData) {
       break;
     }
 
-#elif defined(_M_AMD64)
+#  elif defined(_M_AMD64) || defined(_M_ARM64)
+
+#    if defined(_M_AMD64)
+    auto currentInstr = context->Rip;
+#    elif defined(_M_ARM64)
+    auto currentInstr = context->Pc;
+#    endif
+
     // If we reach a frame in JIT code, we don't have enough information to
     // unwind, so we have to give up.
-    if (sJitCodeRegionStart && (uint8_t*)context->Rip >= sJitCodeRegionStart &&
-        (uint8_t*)context->Rip < sJitCodeRegionStart + sJitCodeRegionSize) {
+    if (sJitCodeRegionStart && (uint8_t*)currentInstr >= sJitCodeRegionStart &&
+        (uint8_t*)currentInstr < sJitCodeRegionStart + sJitCodeRegionSize) {
       break;
     }
 
@@ -333,8 +287,8 @@ static void WalkStackMain64(struct WalkStackData* aData) {
     // unwind data, so their JIT unwind callback just throws up its hands and
     // terminates the process.
     if (sMsMpegJitCodeRegionStart &&
-        (uint8_t*)context->Rip >= sMsMpegJitCodeRegionStart &&
-        (uint8_t*)context->Rip <
+        (uint8_t*)currentInstr >= sMsMpegJitCodeRegionStart &&
+        (uint8_t*)currentInstr <
             sMsMpegJitCodeRegionStart + sMsMpegJitCodeRegionSize) {
       break;
     }
@@ -343,29 +297,39 @@ static void WalkStackMain64(struct WalkStackData* aData) {
     // Try to look up unwind metadata for the current function.
     ULONG64 imageBase;
     PRUNTIME_FUNCTION runtimeFunction =
-        RtlLookupFunctionEntry(context->Rip, &imageBase, NULL);
+        RtlLookupFunctionEntry(currentInstr, &imageBase, NULL);
 
     if (runtimeFunction) {
       PVOID dummyHandlerData;
       ULONG64 dummyEstablisherFrame;
-      RtlVirtualUnwind(UNW_FLAG_NHANDLER, imageBase, context->Rip,
+      RtlVirtualUnwind(UNW_FLAG_NHANDLER, imageBase, currentInstr,
                        runtimeFunction, context, &dummyHandlerData,
                        &dummyEstablisherFrame, nullptr);
     } else if (firstFrame) {
       // Leaf functions can be unwound by hand.
+#    if defined(_M_AMD64)
       context->Rip = *reinterpret_cast<DWORD64*>(context->Rsp);
       context->Rsp += sizeof(void*);
+#    elif defined(_M_ARM64)
+      context->Pc = *reinterpret_cast<DWORD64*>(context->Sp);
+      context->Sp += sizeof(void*);
+#    endif
     } else {
       // Something went wrong.
       break;
     }
 
+#    if defined(_M_AMD64)
     addr = context->Rip;
     spaddr = context->Rsp;
+#    elif defined(_M_ARM64)
+    addr = context->Pc;
+    spaddr = context->Sp;
+#    endif
     firstFrame = false;
-#else
-#error "unknown platform"
-#endif
+#  else
+#    error "unknown platform"
+#  endif
 
     if (addr == 0) {
       break;
@@ -389,65 +353,14 @@ static void WalkStackMain64(struct WalkStackData* aData) {
       break;
     }
 
-#if defined(_M_IX86) || defined(_M_IA64)
+#  if defined(_M_IX86) || defined(_M_IA64)
     if (frame64.AddrReturn.Offset == 0) {
       break;
     }
-#endif
+#  endif
   }
 }
 
-static unsigned int WINAPI WalkStackThread(void* aData) {
-  BOOL msgRet;
-  MSG msg;
-
-  // Call PeekMessage to force creation of a message queue so that
-  // other threads can safely post events to us.
-  ::PeekMessage(&msg, nullptr, WM_USER, WM_USER, PM_NOREMOVE);
-
-  // and tell the thread that created us that we're ready.
-  HANDLE readyEvent = (HANDLE)aData;
-  ::SetEvent(readyEvent);
-
-  while ((msgRet = ::GetMessage(&msg, (HWND)-1, 0, 0)) != 0) {
-    if (msgRet == -1) {
-      PrintError("GetMessage");
-    } else {
-      DWORD ret;
-
-      struct WalkStackData* data = (WalkStackData*)msg.lParam;
-      if (!data) {
-        continue;
-      }
-
-      // Don't suspend the calling thread until it's waiting for
-      // us; otherwise the number of frames on the stack could vary.
-      ret = ::WaitForSingleObject(data->eventStart, INFINITE);
-      if (ret != WAIT_OBJECT_0) {
-        PrintError("WaitForSingleObject");
-      }
-
-      // Suspend the calling thread, dump his stack, and then resume him.
-      // He's currently waiting for us to finish so now should be a good time.
-      ret = ::SuspendThread(data->thread);
-      if (ret == (DWORD)-1) {
-        PrintError("ThreadSuspend");
-      } else {
-        WalkStackMain64(data);
-
-        ret = ::ResumeThread(data->thread);
-        if (ret == (DWORD)-1) {
-          PrintError("ThreadResume");
-        }
-      }
-
-      ::SetEvent(data->eventEnd);
-    }
-  }
-
-  return 0;
-}
-
 /**
  * Walk the stack, translating PC's found into strings and recording the
  * chain in aBuffer. For this to work properly, the DLLs must be rebased
@@ -462,21 +375,20 @@ MFBT_API void MozStackWalkThread(MozWalkStackCallback aCallback,
                                  CONTEXT* aContext) {
   static HANDLE myProcess = nullptr;
   HANDLE myThread;
-  DWORD walkerReturn;
   struct WalkStackData data;
 
   InitializeDbgHelpCriticalSection();
 
-  // EnsureWalkThreadReady's _beginthreadex takes a heap lock and must be
-  // avoided if we're walking another (i.e. suspended) thread.
-  if (!aThread && !EnsureWalkThreadReady()) {
-    return;
+  HANDLE targetThread = aThread;
+  if (!aThread) {
+    targetThread = ::GetCurrentThread();
+    data.walkCallingThread = true;
+  } else {
+    DWORD threadId = ::GetThreadId(aThread);
+    DWORD currentThreadId = ::GetCurrentThreadId();
+    data.walkCallingThread = (threadId == currentThreadId);
   }
 
-  HANDLE currentThread = ::GetCurrentThread();
-  HANDLE targetThread = aThread ? aThread : currentThread;
-  data.walkCallingThread = (targetThread == currentThread);
-
   // Have to duplicate handle to get a real handle.
   if (!myProcess) {
     if (!::DuplicateHandle(::GetCurrentProcess(), ::GetCurrentProcess(),
@@ -511,51 +423,16 @@ MFBT_API void MozStackWalkThread(MozWalkStackCallback aCallback,
   data.sp_size = ArrayLength(local_sps);
   data.context = aContext;
 
-  if (aThread) {
-    // If we're walking the stack of another thread, we don't need to
-    // use a separate walker thread.
+  WalkStackMain64(&data);
+
+  if (data.pc_count > data.pc_size) {
+    data.pcs = (void**)_alloca(data.pc_count * sizeof(void*));
+    data.pc_size = data.pc_count;
+    data.pc_count = 0;
+    data.sps = (void**)_alloca(data.sp_count * sizeof(void*));
+    data.sp_size = data.sp_count;
+    data.sp_count = 0;
     WalkStackMain64(&data);
-
-    if (data.pc_count > data.pc_size) {
-      data.pcs = (void**)_alloca(data.pc_count * sizeof(void*));
-      data.pc_size = data.pc_count;
-      data.pc_count = 0;
-      data.sps = (void**)_alloca(data.sp_count * sizeof(void*));
-      data.sp_size = data.sp_count;
-      data.sp_count = 0;
-      WalkStackMain64(&data);
-    }
-  } else {
-    data.eventStart =
-        ::CreateEvent(nullptr, FALSE /* auto-reset*/,
-                      FALSE /* initially non-signaled */, nullptr);
-    data.eventEnd = ::CreateEvent(nullptr, FALSE /* auto-reset*/,
-                                  FALSE /* initially non-signaled */, nullptr);
-
-    ::PostThreadMessage(gStackWalkThread, WM_USER, 0, (LPARAM)&data);
-
-    walkerReturn =
-        ::SignalObjectAndWait(data.eventStart, data.eventEnd, INFINITE, FALSE);
-    if (walkerReturn != WAIT_OBJECT_0 && data.walkCallingThread) {
-      PrintError("SignalObjectAndWait (1)");
-    }
-    if (data.pc_count > data.pc_size) {
-      data.pcs = (void**)_alloca(data.pc_count * sizeof(void*));
-      data.pc_size = data.pc_count;
-      data.pc_count = 0;
-      data.sps = (void**)_alloca(data.sp_count * sizeof(void*));
-      data.sp_size = data.sp_count;
-      data.sp_count = 0;
-      ::PostThreadMessage(gStackWalkThread, WM_USER, 0, (LPARAM)&data);
-      walkerReturn = ::SignalObjectAndWait(data.eventStart, data.eventEnd,
-                                           INFINITE, FALSE);
-      if (walkerReturn != WAIT_OBJECT_0 && data.walkCallingThread) {
-        PrintError("SignalObjectAndWait (2)");
-      }
-    }
-
-    ::CloseHandle(data.eventStart);
-    ::CloseHandle(data.eventEnd);
   }
 
   ::CloseHandle(myThread);
@@ -618,14 +495,14 @@ static BOOL CALLBACK callbackEspecial64(PCSTR aModuleName, DWORD64 aModuleBase,
 // compiler version, and the version number in debughlp.h was NOT bumped
 // when these changes were made, ifdef based on a constant that was
 // added between these versions.
-#ifdef SSRVOPT_SETCONTEXT
-#define NS_IMAGEHLP_MODULE64_SIZE                                        \
-  (((offsetof(IMAGEHLP_MODULE64, LoadedPdbName) + sizeof(DWORD64) - 1) / \
-    sizeof(DWORD64)) *                                                   \
-   sizeof(DWORD64))
-#else
-#define NS_IMAGEHLP_MODULE64_SIZE sizeof(IMAGEHLP_MODULE64)
-#endif
+#  ifdef SSRVOPT_SETCONTEXT
+#    define NS_IMAGEHLP_MODULE64_SIZE                                        \
+      (((offsetof(IMAGEHLP_MODULE64, LoadedPdbName) + sizeof(DWORD64) - 1) / \
+        sizeof(DWORD64)) *                                                   \
+       sizeof(DWORD64))
+#  else
+#    define NS_IMAGEHLP_MODULE64_SIZE sizeof(IMAGEHLP_MODULE64)
+#  endif
 
 BOOL SymGetModuleInfoEspecial64(HANDLE aProcess, DWORD64 aAddr,
                                 PIMAGEHLP_MODULE64 aModuleInfo,
@@ -775,28 +652,28 @@ MFBT_API bool MozDescribeCodeAddress(void* aPC,
     (HAVE__UNWIND_BACKTRACE || MOZ_STACKWALK_SUPPORTS_LINUX || \
      MOZ_STACKWALK_SUPPORTS_MACOSX)
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
+#  include <stdlib.h>
+#  include <string.h>
+#  include <stdio.h>
 
 // On glibc 2.1, the Dl_info api defined in <dlfcn.h> is only exposed
 // if __USE_GNU is defined.  I suppose its some kind of standards
 // adherence thing.
 //
-#if (__GLIBC_MINOR__ >= 1) && !defined(__USE_GNU)
-#define __USE_GNU
-#endif
+#  if (__GLIBC_MINOR__ >= 1) && !defined(__USE_GNU)
+#    define __USE_GNU
+#  endif
 
 // This thing is exported by libstdc++
 // Yes, this is a gcc only hack
-#if defined(MOZ_DEMANGLE_SYMBOLS)
-#include <cxxabi.h>
-#endif  // MOZ_DEMANGLE_SYMBOLS
+#  if defined(MOZ_DEMANGLE_SYMBOLS)
+#    include <cxxabi.h>
+#  endif  // MOZ_DEMANGLE_SYMBOLS
 
 void DemangleSymbol(const char* aSymbol, char* aBuffer, int aBufLen) {
   aBuffer[0] = '\0';
 
-#if defined(MOZ_DEMANGLE_SYMBOLS)
+#  if defined(MOZ_DEMANGLE_SYMBOLS)
   /* See demangle.h in the gcc source for the voodoo */
   char* demangled = abi::__cxa_demangle(aSymbol, 0, 0, 0);
 
@@ -805,12 +682,12 @@ void DemangleSymbol(const char* aSymbol, char* aBuffer, int aBufLen) {
     aBuffer[aBufLen - 1] = '\0';
     free(demangled);
   }
-#endif  // MOZ_DEMANGLE_SYMBOLS
+#  endif  // MOZ_DEMANGLE_SYMBOLS
 }
 
 // {x86, ppc} x {Linux, Mac} stackwalking code.
-#if ((defined(__i386) || defined(PPC) || defined(__ppc__)) && \
-     (MOZ_STACKWALK_SUPPORTS_MACOSX || MOZ_STACKWALK_SUPPORTS_LINUX))
+#  if ((defined(__i386) || defined(PPC) || defined(__ppc__)) && \
+       (MOZ_STACKWALK_SUPPORTS_MACOSX || MOZ_STACKWALK_SUPPORTS_LINUX))
 
 MFBT_API void MozStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
                            uint32_t aMaxFrames, void* aClosure) {
@@ -818,11 +695,11 @@ MFBT_API void MozStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
   void** bp = (void**)__builtin_frame_address(0);
 
   void* stackEnd;
-#if HAVE___LIBC_STACK_END
+#    if HAVE___LIBC_STACK_END
   stackEnd = __libc_stack_end;
-#elif defined(XP_DARWIN)
+#    elif defined(XP_DARWIN)
   stackEnd = pthread_get_stackaddr_np(pthread_self());
-#elif defined(ANDROID)
+#    elif defined(ANDROID)
   pthread_attr_t sattr;
   pthread_attr_init(&sattr);
   pthread_getattr_np(pthread_self(), &sattr);
@@ -847,17 +724,17 @@ MFBT_API void MozStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
     uintptr_t stackStart = std::max(maxStackStart, uintptr_t(bp));
     stackEnd = reinterpret_cast<void*>(stackStart + kMaxStackSize);
   }
-#else
-#error Unsupported configuration
-#endif
+#    else
+#      error Unsupported configuration
+#    endif
   FramePointerStackWalk(aCallback, aSkipFrames, aMaxFrames, aClosure, bp,
                         stackEnd);
 }
 
-#elif defined(HAVE__UNWIND_BACKTRACE)
+#  elif defined(HAVE__UNWIND_BACKTRACE)
 
 // libgcc_s.so symbols _Unwind_Backtrace@@GCC_3.3 and _Unwind_GetIP@@GCC_3.0
-#include <unwind.h>
+#    include <unwind.h>
 
 struct unwind_info {
   MozWalkStackCallback callback;
@@ -904,7 +781,7 @@ MFBT_API void MozStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
   (void)_Unwind_Backtrace(unwind_callback, &info);
 }
 
-#endif
+#  endif
 
 bool MFBT_API MozDescribeCodeAddress(void* aPC,
                                      MozCodeAddressDetails* aDetails) {
@@ -962,6 +839,7 @@ MFBT_API bool MozDescribeCodeAddress(void* aPC,
 
 #if defined(XP_WIN) || defined(XP_MACOSX) || defined(XP_LINUX)
 namespace mozilla {
+MOZ_ASAN_BLACKLIST
 void FramePointerStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
                            uint32_t aMaxFrames, void* aClosure, void** aBp,
                            void* aStackEnd) {
@@ -977,17 +855,17 @@ void FramePointerStackWalk(MozWalkStackCallback aCallback, uint32_t aSkipFrames,
     // a little if the stack has been corrupted.)
     // We don't need to check against the begining of the stack because
     // we can assume that aBp > sp
-    if (next <= aBp || next > aStackEnd || (uintptr_t(next) & 3)) {
+    if (next <= aBp || next >= aStackEnd || (uintptr_t(next) & 3)) {
       break;
     }
-#if (defined(__ppc__) && defined(XP_MACOSX)) || defined(__powerpc64__)
+#  if (defined(__ppc__) && defined(XP_MACOSX)) || defined(__powerpc64__)
     // ppc mac or powerpc64 linux
     void* pc = *(aBp + 2);
     aBp += 3;
-#else  // i386 or powerpc32 linux
+#  else  // i386 or powerpc32 linux
     void* pc = *(aBp + 1);
     aBp += 2;
-#endif
+#  endif
     if (--skip < 0) {
       // Assume that the SP points to the BP of the function
       // it called. We can't know the exact location of the SP
