@@ -5,38 +5,35 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 // Implement TimeStamp::Now() with QueryPerformanceCounter() controlled with
-// values of GetTickCount().
+// values of GetTickCount64().
 
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/TimeStamp.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <intrin.h>
 #include <windows.h>
 
 // To enable logging define to your favorite logging API
 #define LOG(x)
 
-class AutoCriticalSection
-{
-public:
-  AutoCriticalSection(LPCRITICAL_SECTION aSection)
-    : mSection(aSection)
-  {
+class AutoCriticalSection {
+ public:
+  explicit AutoCriticalSection(LPCRITICAL_SECTION aSection)
+      : mSection(aSection) {
     ::EnterCriticalSection(mSection);
   }
-  ~AutoCriticalSection()
-  {
-    ::LeaveCriticalSection(mSection);
-  }
-private:
+  ~AutoCriticalSection() { ::LeaveCriticalSection(mSection); }
+
+ private:
   LPCRITICAL_SECTION mSection;
 };
 
 // Estimate of the smallest duration of time we can measure.
 static volatile ULONGLONG sResolution;
 static volatile ULONGLONG sResolutionSigDigs;
-static const double   kNsPerSecd  = 1000000000.0;
+static const double kNsPerSecd = 1000000000.0;
 static const LONGLONG kNsPerMillisec = 1000000;
 
 // ----------------------------------------------------------------------------
@@ -62,24 +59,18 @@ static const DWORD kDefaultTimeIncrement = 156001;
 // Global variables, not changing at runtime
 // ----------------------------------------------------------------------------
 
-/**
- * The [mt] unit:
- *
- * Many values are kept in ticks of the Performance Coutner x 1000,
- * further just referred as [mt], meaning milli-ticks.
- *
- * This is needed to preserve maximum precision of the performance frequency
- * representation.  GetTickCount values in milliseconds are multiplied with
- * frequency per second.  Therefor we need to multiply QPC value by 1000 to
- * have the same units to allow simple arithmentic with both QPC and GTC.
- */
-
-#define ms2mt(x) ((x) * sFrequencyPerSec)
-#define mt2ms(x) ((x) / sFrequencyPerSec)
-#define mt2ms_f(x) (double(x) / sFrequencyPerSec)
-
 // Result of QueryPerformanceFrequency
-static LONGLONG sFrequencyPerSec = 0;
+// We use default of 1 for the case we can't use QueryPerformanceCounter
+// to make mt/ms conversions work despite that.
+static uint64_t sFrequencyPerSec = 1;
+
+namespace mozilla {
+
+MFBT_API uint64_t GetQueryPerformanceFrequencyPerSec() {
+  return sFrequencyPerSec;
+}
+
+}  // namespace mozilla
 
 // How much we are tolerant to GTC occasional loose of resoltion.
 // This number says how many multiples of the minimal GTC resolution
@@ -87,17 +78,18 @@ static LONGLONG sFrequencyPerSec = 0;
 static const LONGLONG kGTCTickLeapTolerance = 4;
 
 // Base tolerance (more: "inability of detection" range) threshold is calculated
-// dynamically, and kept in sGTCResulutionThreshold.
+// dynamically, and kept in sGTCResolutionThreshold.
 //
 // Schematically, QPC worked "100%" correctly if ((GTC_now - GTC_epoch) -
-// (QPC_now - QPC_epoch)) was in  [-sGTCResulutionThreshold, sGTCResulutionThreshold]
-// interval every time we'd compared two time stamps.
+// (QPC_now - QPC_epoch)) was in  [-sGTCResolutionThreshold,
+// sGTCResolutionThreshold] interval every time we'd compared two time stamps.
 // If not, then we check the overflow behind this basic threshold
-// is in kFailureThreshold.  If not, we condider it as a QPC failure.  If too many
-// failures in short time are detected, QPC is considered faulty and disabled.
+// is in kFailureThreshold.  If not, we condider it as a QPC failure.  If too
+// many failures in short time are detected, QPC is considered faulty and
+// disabled.
 //
 // Kept in [mt]
-static LONGLONG sGTCResulutionThreshold;
+static LONGLONG sGTCResolutionThreshold;
 
 // If QPC is found faulty for two stamps in this interval, we engage
 // the fault detection algorithm.  For duration larger then this limit
@@ -156,57 +148,29 @@ static CRITICAL_SECTION sTimeStampLock;
 // Kept in [mt]
 static ULONGLONG sFaultIntoleranceCheckpoint = 0;
 
-// Used only when GetTickCount64 is not available on the platform.
-// Last result of GetTickCount call.
-//
-// Kept in [ms]
-static DWORD sLastGTCResult = 0;
-
-// Higher part of the 64-bit value of MozGetTickCount64,
-// incremented atomically.
-static DWORD sLastGTCRollover = 0;
-
 namespace mozilla {
 
-typedef ULONGLONG (WINAPI* GetTickCount64_t)();
-static GetTickCount64_t sGetTickCount64 = nullptr;
-
-// Function protecting GetTickCount result from rolling over,
-// result is in [ms]
-static ULONGLONG WINAPI
-MozGetTickCount64()
-{
-  DWORD GTC = ::GetTickCount();
-
-  // Cheaper then CMPXCHG8B
-  AutoCriticalSection lock(&sTimeStampLock);
-
-  // Pull the rollover counter forward only if new value of GTC goes way
-  // down under the last saved result
-  if ((sLastGTCResult > GTC) && ((sLastGTCResult - GTC) > (1UL << 30))) {
-    ++sLastGTCRollover;
-  }
-
-  sLastGTCResult = GTC;
-  return ULONGLONG(sLastGTCRollover) << 32 | sLastGTCResult;
-}
-
 // Result is in [mt]
-static inline ULONGLONG
-PerformanceCounter()
-{
+static inline ULONGLONG PerformanceCounter() {
   LARGE_INTEGER pc;
   ::QueryPerformanceCounter(&pc);
+
+  // QueryPerformanceCounter may slightly jitter (not be 100% monotonic.)
+  // This is a simple go-backward protection for such a faulty hardware.
+  AutoCriticalSection lock(&sTimeStampLock);
+
+  static decltype(LARGE_INTEGER::QuadPart) last;
+  if (last > pc.QuadPart) {
+    return last * 1000ULL;
+  }
+  last = pc.QuadPart;
   return pc.QuadPart * 1000ULL;
 }
 
-static void
-InitThresholds()
-{
+static void InitThresholds() {
   DWORD timeAdjustment = 0, timeIncrement = 0;
   BOOL timeAdjustmentDisabled;
-  GetSystemTimeAdjustment(&timeAdjustment,
-                          &timeIncrement,
+  GetSystemTimeAdjustment(&timeAdjustment, &timeIncrement,
                           &timeAdjustmentDisabled);
 
   LOG(("TimeStamp: timeIncrement=%d [100ns]", timeIncrement));
@@ -229,38 +193,41 @@ InitThresholds()
 
   // How many milli-ticks has the interval rounded up
   LONGLONG ticksPerGetTickCountResolutionCeiling =
-    (int64_t(timeIncrementCeil) * sFrequencyPerSec) / 10000LL;
+      (int64_t(timeIncrementCeil) * sFrequencyPerSec) / 10000LL;
 
   // GTC may jump by 32 (2*16) ms in two steps, therefor use the ceiling value.
-  sGTCResulutionThreshold =
-    LONGLONG(kGTCTickLeapTolerance * ticksPerGetTickCountResolutionCeiling);
+  sGTCResolutionThreshold =
+      LONGLONG(kGTCTickLeapTolerance * ticksPerGetTickCountResolutionCeiling);
 
   sHardFailureLimit = ms2mt(kHardFailureLimit);
   sFailureFreeInterval = ms2mt(kFailureFreeInterval);
   sFailureThreshold = ms2mt(kFailureThreshold);
 }
 
-static void
-InitResolution()
-{
+static void InitResolution() {
   // 10 total trials is arbitrary: what we're trying to avoid by
   // looping is getting unlucky and being interrupted by a context
   // switch or signal, or being bitten by paging/cache effects
 
   ULONGLONG minres = ~0ULL;
-  int loops = 10;
-  do {
-    ULONGLONG start = PerformanceCounter();
-    ULONGLONG end = PerformanceCounter();
+  if (sUseQPC) {
+    int loops = 10;
+    do {
+      ULONGLONG start = PerformanceCounter();
+      ULONGLONG end = PerformanceCounter();
 
-    ULONGLONG candidate = (end - start);
-    if (candidate < minres) {
-      minres = candidate;
+      ULONGLONG candidate = (end - start);
+      if (candidate < minres) {
+        minres = candidate;
+      }
+    } while (--loops && minres);
+
+    if (0 == minres) {
+      minres = 1;
     }
-  } while (--loops && minres);
-
-  if (0 == minres) {
-    minres = 1;
+  } else {
+    // GetTickCount has only ~16ms known resolution
+    minres = ms2mt(16);
   }
 
   // Converting minres that is in [mt] to nanosecods, multiplicating
@@ -275,9 +242,9 @@ InitResolution()
   // find the number of significant digits in mResolution, for the
   // sake of ToSecondsSigDigits()
   ULONGLONG sigDigs;
-  for (sigDigs = 1;
-       !(sigDigs == result || 10 * sigDigs > result);
-       sigDigs *= 10);
+  for (sigDigs = 1; !(sigDigs == result || 10 * sigDigs > result);
+       sigDigs *= 10)
+    ;
 
   sResolutionSigDigs = sigDigs;
 }
@@ -286,25 +253,22 @@ InitResolution()
 // TimeStampValue implementation
 // ----------------------------------------------------------------------------
 MFBT_API
-TimeStampValue::TimeStampValue(ULONGLONG aGTC, ULONGLONG aQPC, bool aHasQPC)
-  : mGTC(aGTC)
-  , mQPC(aQPC)
-  , mHasQPC(aHasQPC)
-  , mIsNull(false)
-{
+TimeStampValue::TimeStampValue(ULONGLONG aGTC, ULONGLONG aQPC, bool aHasQPC,
+                               bool aUsedCanonicalNow)
+    : mGTC(aGTC),
+      mQPC(aQPC),
+      mUsedCanonicalNow(aUsedCanonicalNow),
+      mHasQPC(aHasQPC) {
+  mIsNull = aGTC == 0 && aQPC == 0;
 }
 
-MFBT_API TimeStampValue&
-TimeStampValue::operator+=(const int64_t aOther)
-{
+MFBT_API TimeStampValue& TimeStampValue::operator+=(const int64_t aOther) {
   mGTC += aOther;
   mQPC += aOther;
   return *this;
 }
 
-MFBT_API TimeStampValue&
-TimeStampValue::operator-=(const int64_t aOther)
-{
+MFBT_API TimeStampValue& TimeStampValue::operator-=(const int64_t aOther) {
   mGTC -= aOther;
   mQPC -= aOther;
   return *this;
@@ -312,30 +276,28 @@ TimeStampValue::operator-=(const int64_t aOther)
 
 // If the duration is less then two seconds, perform check of QPC stability
 // by comparing both GTC and QPC calculated durations of this and aOther.
-MFBT_API uint64_t
-TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
-{
+MFBT_API uint64_t TimeStampValue::CheckQPC(const TimeStampValue& aOther) const {
   uint64_t deltaGTC = mGTC - aOther.mGTC;
 
-  if (!mHasQPC || !aOther.mHasQPC) { // Both not holding QPC
+  if (!mHasQPC || !aOther.mHasQPC) {  // Both not holding QPC
     return deltaGTC;
   }
 
   uint64_t deltaQPC = mQPC - aOther.mQPC;
 
-  if (sHasStableTSC) { // For stable TSC there is no need to check
+  if (sHasStableTSC) {  // For stable TSC there is no need to check
     return deltaQPC;
   }
 
   // Check QPC is sane before using it.
   int64_t diff = DeprecatedAbs(int64_t(deltaQPC) - int64_t(deltaGTC));
-  if (diff <= sGTCResulutionThreshold) {
+  if (diff <= sGTCResolutionThreshold) {
     return deltaQPC;
   }
 
   // Treat absolutely for calibration purposes
   int64_t duration = DeprecatedAbs(int64_t(deltaGTC));
-  int64_t overflow = diff - sGTCResulutionThreshold;
+  int64_t overflow = diff - sGTCResolutionThreshold;
 
   LOG(("TimeStamp: QPC check after %llums with overflow %1.4fms",
        mt2ms(duration), mt2ms_f(overflow)));
@@ -346,7 +308,8 @@ TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
 
   // QPC deviates, don't use it, since now this method may only return deltaGTC.
 
-  if (!sUseQPC) { // QPC already disabled, no need to run the fault tolerance algorithm.
+  if (!sUseQPC) {  // QPC already disabled, no need to run the fault tolerance
+                   // algorithm.
     return deltaGTC;
   }
 
@@ -355,7 +318,7 @@ TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
   if (duration < sHardFailureLimit) {
     // Interval between the two time stamps is very short, consider
     // QPC as unstable and record a failure.
-    uint64_t now = ms2mt(sGetTickCount64());
+    uint64_t now = ms2mt(GetTickCount64());
 
     AutoCriticalSection lock(&sTimeStampLock);
 
@@ -364,8 +327,8 @@ TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
       // Time since now to the checkpoint actually holds information on how many
       // failures there were in the failure free interval we have defined.
       uint64_t failureCount =
-        (sFaultIntoleranceCheckpoint - now + sFailureFreeInterval - 1) /
-        sFailureFreeInterval;
+          (sFaultIntoleranceCheckpoint - now + sFailureFreeInterval - 1) /
+          sFailureFreeInterval;
       if (failureCount > kMaxFailuresPerInterval) {
         sUseQPC = false;
         LOG(("TimeStamp: QPC disabled"));
@@ -377,7 +340,8 @@ TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
         LOG(("TimeStamp: recording %dth QPC failure", failureCount));
       }
     } else {
-      // Setup fault intolerance checkpoint in the future for first detected error.
+      // Setup fault intolerance checkpoint in the future for first detected
+      // error.
       sFaultIntoleranceCheckpoint = now + sFailureFreeInterval;
       LOG(("TimeStamp: recording 1st QPC failure"));
     }
@@ -387,9 +351,8 @@ TimeStampValue::CheckQPC(const TimeStampValue& aOther) const
 }
 
 MFBT_API uint64_t
-TimeStampValue::operator-(const TimeStampValue& aOther) const
-{
-  if (mIsNull && aOther.mIsNull) {
+TimeStampValue::operator-(const TimeStampValue& aOther) const {
+  if (IsNull() && aOther.IsNull()) {
     return uint64_t(0);
   }
 
@@ -400,16 +363,13 @@ TimeStampValue::operator-(const TimeStampValue& aOther) const
 // TimeDuration and TimeStamp implementation
 // ----------------------------------------------------------------------------
 
-MFBT_API double
-BaseTimeDurationPlatformUtils::ToSeconds(int64_t aTicks)
-{
+MFBT_API double BaseTimeDurationPlatformUtils::ToSeconds(int64_t aTicks) {
   // Converting before arithmetic avoids blocked store forward
   return double(aTicks) / (double(sFrequencyPerSec) * 1000.0);
 }
 
-MFBT_API double
-BaseTimeDurationPlatformUtils::ToSecondsSigDigits(int64_t aTicks)
-{
+MFBT_API double BaseTimeDurationPlatformUtils::ToSecondsSigDigits(
+    int64_t aTicks) {
   // don't report a value < mResolution ...
   LONGLONG resolution = sResolution;
   LONGLONG resolutionSigDigs = sResolutionSigDigs;
@@ -420,8 +380,7 @@ BaseTimeDurationPlatformUtils::ToSecondsSigDigits(int64_t aTicks)
 }
 
 MFBT_API int64_t
-BaseTimeDurationPlatformUtils::TicksFromMilliseconds(double aMilliseconds)
-{
+BaseTimeDurationPlatformUtils::TicksFromMilliseconds(double aMilliseconds) {
   double result = ms2mt(aMilliseconds);
   if (result > INT64_MAX) {
     return INT64_MAX;
@@ -432,31 +391,31 @@ BaseTimeDurationPlatformUtils::TicksFromMilliseconds(double aMilliseconds)
   return result;
 }
 
-MFBT_API int64_t
-BaseTimeDurationPlatformUtils::ResolutionInTicks()
-{
+MFBT_API int64_t BaseTimeDurationPlatformUtils::ResolutionInTicks() {
   return static_cast<int64_t>(sResolution);
 }
 
-static bool
-HasStableTSC()
-{
-  union
-  {
+static bool HasStableTSC() {
+#if defined(_M_ARM64)
+  // AArch64 defines that its system counter run at a constant rate
+  // regardless of the current clock frequency of the system.  See "The
+  // Generic Timer", section D7, in the ARMARM for ARMv8.
+  return true;
+#else
+  union {
     int regs[4];
-    struct
-    {
+    struct {
       int nIds;
       char cpuString[12];
     };
   } cpuInfo;
 
   __cpuid(cpuInfo.regs, 0);
-  // Only allow Intel CPUs for now
+  // Only allow Intel or AMD CPUs for now.
   // The order of the registers is reg[1], reg[3], reg[2].  We just adjust the
   // string so that we can compare in one go.
-  if (_strnicmp(cpuInfo.cpuString, "GenuntelineI",
-                sizeof(cpuInfo.cpuString))) {
+  if (_strnicmp(cpuInfo.cpuString, "GenuntelineI", sizeof(cpuInfo.cpuString)) &&
+      _strnicmp(cpuInfo.cpuString, "AuthcAMDenti", sizeof(cpuInfo.cpuString))) {
     return false;
   }
 
@@ -464,21 +423,22 @@ HasStableTSC()
 
   // detect if the Advanced Power Management feature is supported
   __cpuid(regs, 0x80000000);
-  if (regs[0] < 0x80000007) {
+  if ((unsigned int)regs[0] < 0x80000007) {
+    // XXX should we return true here?  If there is no APM there may be
+    // no way how TSC can run out of sync among cores.
     return false;
   }
 
   __cpuid(regs, 0x80000007);
   // if bit 8 is set than TSC will run at a constant rate
-  // in all ACPI P-state, C-states and T-states
+  // in all ACPI P-states, C-states and T-states
   return regs[3] & (1 << 8);
+#endif
 }
 
 static bool gInitialized = false;
 
-MFBT_API void
-TimeStamp::Startup()
-{
+MFBT_API void TimeStamp::Startup() {
   if (gInitialized) {
     return;
   }
@@ -487,30 +447,33 @@ TimeStamp::Startup()
 
   // Decide which implementation to use for the high-performance timer.
 
-  HMODULE kernelDLL = GetModuleHandleW(L"kernel32.dll");
-  sGetTickCount64 = reinterpret_cast<GetTickCount64_t>(
-    GetProcAddress(kernelDLL, "GetTickCount64"));
-  if (!sGetTickCount64) {
-    // If the platform does not support the GetTickCount64 (Windows XP doesn't),
-    // then use our fallback implementation based on GetTickCount.
-    sGetTickCount64 = MozGetTickCount64;
-  }
-
   InitializeCriticalSectionAndSpinCount(&sTimeStampLock, kLockSpinCount);
 
-  sHasStableTSC = HasStableTSC();
-  LOG(("TimeStamp: HasStableTSC=%d", sHasStableTSC));
+  bool forceGTC = false;
+  bool forceQPC = false;
+
+  char* modevar = getenv("MOZ_TIMESTAMP_MODE");
+  if (modevar) {
+    if (!strcmp(modevar, "QPC")) {
+      forceQPC = true;
+    } else if (!strcmp(modevar, "GTC")) {
+      forceGTC = true;
+    }
+  }
 
   LARGE_INTEGER freq;
-  sUseQPC = ::QueryPerformanceFrequency(&freq);
+  sUseQPC = !forceGTC && ::QueryPerformanceFrequency(&freq);
   if (!sUseQPC) {
-    // No Performance Counter.  Fall back to use GetTickCount.
+    // No Performance Counter.  Fall back to use GetTickCount64.
     InitResolution();
 
-    LOG(("TimeStamp: using GetTickCount"));
+    LOG(("TimeStamp: using GetTickCount64"));
     return;
   }
 
+  sHasStableTSC = forceQPC || HasStableTSC();
+  LOG(("TimeStamp: HasStableTSC=%d", sHasStableTSC));
+
   sFrequencyPerSec = freq.QuadPart;
   LOG(("TimeStamp: QPC frequency=%llu", sFrequencyPerSec));
 
@@ -520,30 +483,30 @@ TimeStamp::Startup()
   return;
 }
 
-MFBT_API void
-TimeStamp::Shutdown()
-{
-  DeleteCriticalSection(&sTimeStampLock);
-}
+MFBT_API void TimeStamp::Shutdown() { DeleteCriticalSection(&sTimeStampLock); }
 
-MFBT_API TimeStamp
-TimeStamp::Now(bool aHighResolution)
-{
+TimeStampValue NowInternal(bool aHighResolution) {
   // sUseQPC is volatile
   bool useQPC = (aHighResolution && sUseQPC);
 
   // Both values are in [mt] units.
   ULONGLONG QPC = useQPC ? PerformanceCounter() : uint64_t(0);
-  ULONGLONG GTC = ms2mt(sGetTickCount64());
-  return TimeStamp(TimeStampValue(GTC, QPC, useQPC));
+  ULONGLONG GTC = ms2mt(GetTickCount64());
+  return TimeStampValue(GTC, QPC, useQPC, false);
+}
+
+MFBT_API TimeStamp TimeStamp::Now(bool aHighResolution) {
+  return TimeStamp::NowFuzzy(NowInternal(aHighResolution));
+}
+
+MFBT_API TimeStamp TimeStamp::NowUnfuzzed(bool aHighResolution) {
+  return TimeStamp(NowInternal(aHighResolution));
 }
 
 // Computes and returns the process uptime in microseconds.
 // Returns 0 if an error was encountered.
 
-MFBT_API uint64_t
-TimeStamp::ComputeProcessUptime()
-{
+MFBT_API uint64_t TimeStamp::ComputeProcessUptime() {
   SYSTEMTIME nowSys;
   GetSystemTime(&nowSys);
 
@@ -561,16 +524,10 @@ TimeStamp::ComputeProcessUptime()
     return 0;
   }
 
-  ULARGE_INTEGER startUsec = {{
-     start.dwLowDateTime,
-     start.dwHighDateTime
-  }};
-  ULARGE_INTEGER nowUsec = {{
-    now.dwLowDateTime,
-    now.dwHighDateTime
-  }};
+  ULARGE_INTEGER startUsec = {{start.dwLowDateTime, start.dwHighDateTime}};
+  ULARGE_INTEGER nowUsec = {{now.dwLowDateTime, now.dwHighDateTime}};
 
   return (nowUsec.QuadPart - startUsec.QuadPart) / 10ULL;
 }
 
-} // namespace mozilla
+}  // namespace mozilla
