@@ -567,53 +567,67 @@ class TestPreprocessor(unittest.TestCase):
 
     def test_include_line(self):
         files = {
-            'test.js': '\n'.join([
+            'srcdir/test.js': '\n'.join([
                 '#define foo foobarbaz',
                 '#include @inc@',
                 '@bar@',
                 '',
             ]),
-            'bar.js': '\n'.join([
+            'srcdir/bar.js': '\n'.join([
                 '#define bar barfoobaz',
                 '@foo@',
                 '',
             ]),
-            'foo.js': '\n'.join([
+            'srcdir/foo.js': '\n'.join([
                 'bazfoobar',
                 '#include bar.js',
                 'bazbarfoo',
                 '',
             ]),
-            'baz.js': 'baz\n',
-            'f.js': '\n'.join([
+            'objdir/baz.js': 'baz\n',
+            'srcdir/f.js': '\n'.join([
                 '#include foo.js',
                 '#filter substitution',
                 '#define inc bar.js',
                 '#include test.js',
-                '#include baz.js',
+                '#include ../objdir/baz.js',
                 'fin',
                 '',
             ]),
         }
 
+        preprocessed = ('//@line 1 "$SRCDIR/foo.js"\n'
+                        'bazfoobar\n'
+                        '//@line 2 "$SRCDIR/bar.js"\n'
+                        '@foo@\n'
+                        '//@line 3 "$SRCDIR/foo.js"\n'
+                        'bazbarfoo\n'
+                        '//@line 2 "$SRCDIR/bar.js"\n'
+                        'foobarbaz\n'
+                        '//@line 3 "$SRCDIR/test.js"\n'
+                        'barfoobaz\n'
+                        '//@line 1 "$OBJDIR/baz.js"\n'
+                        'baz\n'
+                        '//@line 6 "$SRCDIR/f.js"\n'
+                        'fin\n').replace('DIR/', 'DIR' + os.sep)
+
+        # Try with separate srcdir/objdir
         with MockedOpen(files):
-            self.pp.do_include('f.js')
-            self.assertEqual(self.pp.out.getvalue(),
-                             ('//@line 1 "CWD/foo.js"\n'
-                              'bazfoobar\n'
-                              '//@line 2 "CWD/bar.js"\n'
-                              '@foo@\n'
-                              '//@line 3 "CWD/foo.js"\n'
-                              'bazbarfoo\n'
-                              '//@line 2 "CWD/bar.js"\n'
-                              'foobarbaz\n'
-                              '//@line 3 "CWD/test.js"\n'
-                              'barfoobaz\n'
-                              '//@line 1 "CWD/baz.js"\n'
-                              'baz\n'
-                              '//@line 6 "CWD/f.js"\n'
-                              'fin\n').replace('CWD/',
-                                               os.getcwd() + os.path.sep))
+            self.pp.topsrcdir = os.path.abspath('srcdir')
+            self.pp.topobjdir = os.path.abspath('objdir')
+            self.pp.do_include('srcdir/f.js')
+            self.assertEqual(self.pp.out.getvalue(), preprocessed)
+
+        # Try again with relative objdir
+        self.setUp()
+        files['srcdir/objdir/baz.js'] = files['objdir/baz.js']
+        del files['objdir/baz.js']
+        files['srcdir/f.js'] = files['srcdir/f.js'].replace('../', '')
+        with MockedOpen(files):
+            self.pp.topsrcdir = os.path.abspath('srcdir')
+            self.pp.topobjdir = os.path.abspath('srcdir/objdir')
+            self.pp.do_include('srcdir/f.js')
+            self.assertEqual(self.pp.out.getvalue(), preprocessed)
 
     def test_include_missing_file(self):
         with MockedOpen({'f': '#include foo\n'}):
