@@ -41,7 +41,8 @@ class VendorRust(MozbuildObject):
         '''
         Ensure that cargo-vendor is new enough. cargo-vendor 0.1.13 and newer
         strips out .cargo-ok, .orig and .rej files, and deals with [patch]
-        replacements in Cargo.toml files which we want.
+        replacements in Cargo.toml files which we want.  Version 0.1.14 and up
+        handles local modifications to vendored crates (bug 1323557).
         '''
         for l in subprocess.check_output([cargo, 'install', '--list']).splitlines():
             # The line looks like one of the following:
@@ -51,7 +52,7 @@ class VendorRust(MozbuildObject):
             m = re.match('cargo-vendor v((\d+\.)*\d+)', l)
             if m:
                 version = m.group(1)
-                return LooseVersion(version) >= b'0.1.13'
+                return LooseVersion(version) >= b'0.1.23'
         return False
 
     def check_modified_files(self):
@@ -118,7 +119,7 @@ Please commit or stash these changes before vendoring, or re-run with `--ignore-
             self.run_process(args=[cargo, 'install', 'cargo-vendor'],
                              append_env=env)
         elif not self.check_cargo_vendor_version(cargo):
-            self.log(logging.INFO, 'cargo_vendor', {}, 'cargo-vendor >= 0.1.12 required; force-reinstalling (this may take a few minutes)...')
+            self.log(logging.INFO, 'cargo_vendor', {}, 'cargo-vendor >= 0.1.23 required; force-reinstalling (this may take a few minutes)...')
             env = self.check_openssl()
             self.run_process(args=[cargo, 'install', '--force', 'cargo-vendor'],
                              append_env=env)
@@ -127,51 +128,125 @@ Please commit or stash these changes before vendoring, or re-run with `--ignore-
 
         return cargo
 
-    def _check_licenses(self, vendor_dir):
-        # A whitelist of acceptable license identifiers for the
-        # packages.license field from https://spdx.org/licenses/.  Cargo
-        # documentation claims that values are checked against the above
-        # list and that multiple entries can be separated by '/'.  We
-        # choose to list all combinations instead for the sake of
-        # completeness and because some entries below obviously do not
-        # conform to the format prescribed in the documentation.
-        #
-        # It is insufficient to have additions to this whitelist reviewed
-        # solely by a build peer; any additions must be checked by somebody
-        # competent to review licensing minutiae.
-        LICENSE_WHITELIST = [
-            'Apache-2.0',
-            'Apache-2.0 / MIT',
-            'Apache-2.0/MIT',
-            'Apache-2 / MIT',
-            'BSD-3-Clause', # bindgen (only used at build time)
-            'CC0-1.0',
-            'ISC',
-            'ISC/Apache-2.0',
-            'MIT',
-            'MIT / Apache-2.0',
-            'MIT/Apache-2.0',
-            'MIT OR Apache-2.0',
-            'MPL-2.0',
-            'Unlicense/MIT',
+    # A whitelist of acceptable license identifiers for the
+    # packages.license field from https://spdx.org/licenses/.  Cargo
+    # documentation claims that values are checked against the above
+    # list and that multiple entries can be separated by '/'.  We
+    # choose to list all combinations instead for the sake of
+    # completeness and because some entries below obviously do not
+    # conform to the format prescribed in the documentation.
+    #
+    # It is insufficient to have additions to this whitelist reviewed
+    # solely by a build peer; any additions must be checked by somebody
+    # competent to review licensing minutiae.
+
+    # Licenses for code used at runtime. Please see the above comment before
+    # adding anything to this list.
+    RUNTIME_LICENSE_WHITELIST = [
+        'Apache-2.0',
+        'Apache-2.0 WITH LLVM-exception',
+        'BSD-2-Clause',
+        # BSD-3-Clause is ok, but packages using it must be added to the
+        # appropriate section of about:licenses. To encourage people to remember
+        # to do that, we do not whitelist the license itself and we require the
+        # packages to be added to RUNTIME_LICENSE_PACKAGE_WHITELIST below.
+        'CC0-1.0',
+        'ISC',
+        'MIT',
+        'MPL-2.0',
+        'Unlicense',
+    ]
+
+    # Licenses for code used at build time (e.g. code generators). Please see the above
+    # comments before adding anything to this list.
+    BUILDTIME_LICENSE_WHITELIST = {
+        'BSD-3-Clause': [
+            'adler32',
+            'bindgen',
+            'fuchsia-zircon',
+            'fuchsia-zircon-sys',
+            'fuchsia-cprng',
         ]
+    }
 
-        # This whitelist should only be used for packages that use a
-        # license-file and for which the license-file entry has been
-        # reviewed.  The table is keyed by package names and maps to the
-        # sha256 hash of the license file that we reviewed.
-        #
-        # As above, it is insufficient to have additions to this whitelist
-        # reviewed solely by a build peer; any additions must be checked by
-        # somebody competent to review licensing minutiae.
-        LICENSE_FILE_PACKAGE_WHITELIST = {
-            # MIT
-            'deque': '6485b8ed310d3f0340bf1ad1f47645069ce4069dcc6bb46c7d5c6faf41de1fdb',
-        }
+    # This whitelist should only be used for packages that use an acceptable
+    # license, but that also need to explicitly mentioned in about:license.
+    RUNTIME_LICENSE_PACKAGE_WHITELIST = {
+        'BSD-3-Clause': [
+        ]
+    }
 
+    # This whitelist should only be used for packages that use a
+    # license-file and for which the license-file entry has been
+    # reviewed.  The table is keyed by package names and maps to the
+    # sha256 hash of the license file that we reviewed.
+    #
+    # As above, it is insufficient to have additions to this whitelist
+    # reviewed solely by a build peer; any additions must be checked by
+    # somebody competent to review licensing minutiae.
+    RUNTIME_LICENSE_FILE_PACKAGE_WHITELIST = {
+        # MIT
+        'deque': '6485b8ed310d3f0340bf1ad1f47645069ce4069dcc6bb46c7d5c6faf41de1fdb',
+    }
+
+    @staticmethod
+    def runtime_license(package, license_string):
+        """Cargo docs say:
+        ---
+        https://doc.rust-lang.org/cargo/reference/manifest.html
+
+        This is an SPDX 2.1 license expression for this package.  Currently
+        crates.io will validate the license provided against a whitelist of
+        known license and exception identifiers from the SPDX license list
+        2.4.  Parentheses are not currently supported.
+
+        Multiple licenses can be separated with a `/`, although that usage
+        is deprecated.  Instead, use a license expression with AND and OR
+        operators to get more explicit semantics.
+        ---
+        But I have no idea how you can meaningfully AND licenses, so
+        we will abort if that is detected. We'll handle `/` and OR as
+        equivalent and approve is any is in our approved list."""
+
+        if re.search(r'\s+AND', license_string):
+            return False
+
+        license_list = re.split(r'\s*/\s*|\s+OR\s+', license_string)
+        for license in license_list:
+            if license in VendorRust.RUNTIME_LICENSE_WHITELIST:
+                return True
+            if package in VendorRust.RUNTIME_LICENSE_PACKAGE_WHITELIST.get(license, []):
+                return True
+        return False
+
+    def _check_licenses(self, vendor_dir):
         LICENSE_LINE_RE = re.compile(r'\s*license\s*=\s*"([^"]+)"')
         LICENSE_FILE_LINE_RE = re.compile(r'\s*license[-_]file\s*=\s*"([^"]+)"')
 
+        def verify_acceptable_license(package, license):
+            self.log(logging.DEBUG, 'package_license', {},
+                     'has license {}'.format(license))
+
+            if not self.runtime_license(package, license):
+                if license not in self.BUILDTIME_LICENSE_WHITELIST:
+                    self.log(logging.ERROR, 'package_license_error', {},
+                            '''Package {} has a non-approved license: {}.
+
+    Please request license review on the package's license.  If the package's license
+    is approved, please add it to the whitelist of suitable licenses.
+    '''.format(package, license))
+                    return False
+                elif package not in self.BUILDTIME_LICENSE_WHITELIST[license]:
+                    self.log(logging.ERROR, 'package_license_error', {},
+                            '''Package {} has a license that is approved for build-time dependencies: {}
+    but the package itself is not whitelisted as being a build-time only package.
+
+    If your package is build-time only, please add it to the whitelist of build-time
+    only packages. Otherwise, you need to request license review on the package's license.
+    If the package's license is approved, please add it to the whitelist of suitable licenses.
+    '''.format(package, license))
+                    return False
+
         def check_package(package):
             self.log(logging.DEBUG, 'package_check', {},
                      'Checking license for {}'.format(package))
@@ -204,23 +279,13 @@ Please commit or stash these changes before vendoring, or re-run with `--ignore-
 
                 if license_matches:
                     license = license_matches[0].group(1)
-                    self.log(logging.DEBUG, 'package_license', {},
-                             'has license {}'.format(license))
-
-                    if license not in LICENSE_WHITELIST:
-                        self.log(logging.ERROR, 'package_license_error', {},
-                                 '''Package {} has a non-approved license: {}.
-
-Please request license review on the package's license.  If the package's license
-is approved, please add it to the whitelist of suitable licenses.
-'''.format(package, license))
-                        return False
+                    verify_acceptable_license(package, license)
                 else:
                     license_file = license_file_matches[0].group(1)
                     self.log(logging.DEBUG, 'package_license_file', {},
                              'has license-file {}'.format(license_file))
 
-                    if package not in LICENSE_FILE_PACKAGE_WHITELIST:
+                    if package not in self.RUNTIME_LICENSE_FILE_PACKAGE_WHITELIST:
                         self.log(logging.ERROR, 'package_license_file_unknown', {},
                                  '''Package {} has an unreviewed license file: {}.
 
@@ -229,7 +294,7 @@ to the whitelist of packages whose licenses are suitable.
 '''.format(package, license_file))
                         return False
 
-                    approved_hash = LICENSE_FILE_PACKAGE_WHITELIST[package]
+                    approved_hash = self.RUNTIME_LICENSE_FILE_PACKAGE_WHITELIST[package]
                     license_contents = open(os.path.join(vendor_dir, package, license_file), 'r').read()
                     current_hash = hashlib.sha256(license_contents).hexdigest()
                     if current_hash != approved_hash:
@@ -246,7 +311,7 @@ license file's hash.
         # Force all of the packages to be checked for license information
         # before reducing via `all`, so all license issues are found in a
         # single `mach vendor rust` invocation.
-        results = [check_package(p) for p in os.listdir(vendor_dir)]
+        results = [check_package(p) for p in os.listdir(vendor_dir) if os.path.isdir(os.path.join(vendor_dir, p))]
         return all(results)
 
     def vendor(self, ignore_modified=False,
@@ -262,14 +327,12 @@ license file's hash.
 
         relative_vendor_dir = 'third_party/rust'
         vendor_dir = mozpath.join(self.topsrcdir, relative_vendor_dir)
-        self.log(logging.INFO, 'rm_vendor_dir', {}, 'rm -rf %s' % vendor_dir)
-        mozfile.remove(vendor_dir)
 
         # We use check_call instead of mozprocess to ensure errors are displayed.
         # We do an |update -p| here to regenerate the Cargo.lock file with minimal changes. See bug 1324462
         subprocess.check_call([cargo, 'update', '-p', 'gkrust'], cwd=self.topsrcdir)
 
-        subprocess.check_call([cargo, 'vendor', '--quiet', '--no-delete', '--sync', 'Cargo.lock'] + [vendor_dir], cwd=self.topsrcdir)
+        subprocess.check_call([cargo, 'vendor', '--quiet', '--sync', 'Cargo.lock'] + [vendor_dir], cwd=self.topsrcdir)
 
         if not self._check_licenses(vendor_dir):
             self.log(logging.ERROR, 'license_check_failed', {},
