@@ -23,17 +23,15 @@ from mozpack.chrome.manifest import (
     ManifestLocale,
 )
 from mozpack.errors import (
-    errors,
     ErrorMessage,
 )
 from mozpack.test.test_files import (
-    MockDest,
     foo_xpt,
     foo2_xpt,
     bar_xpt,
-    read_interfaces,
 )
 import mozpack.path as mozpath
+from itertools import chain
 from test_errors import TestErrors
 
 
@@ -44,6 +42,7 @@ CONTENTS = {
         'app': False,
         'addon0': 'unpacked',
         'addon1': True,
+        'app/chrome/addons/addon2': True,
     },
     'manifests': [
         ManifestContent('chrome/f', 'oo', 'oo/'),
@@ -52,8 +51,9 @@ CONTENTS = {
         ManifestBinaryComponent('components', 'foo.so'),
         ManifestContent('app/chrome', 'content', 'foo/'),
         ManifestComponent('app/components', '{foo-id}', 'foo.js'),
-        ManifestContent('addon0/chrome', 'content', 'foo/bar/'),
-        ManifestContent('addon1/chrome', 'content', 'foo/bar/'),
+        ManifestContent('addon0/chrome', 'addon0', 'foo/bar/'),
+        ManifestContent('addon1/chrome', 'addon1', 'foo/bar/'),
+        ManifestContent('app/chrome/addons/addon2/chrome', 'addon2', 'foo/bar/'),
     ],
     'files': {
         'chrome/f/oo/bar/baz': GeneratedFile('foobarbaz'),
@@ -71,6 +71,9 @@ CONTENTS = {
         'addon1/chrome/foo/bar/baz': GeneratedFile('foobarbaz'),
         'addon1/components/foo.xpt': foo2_xpt,
         'addon1/components/bar.xpt': bar_xpt,
+        'app/chrome/addons/addon2/chrome/foo/bar/baz': GeneratedFile('foobarbaz'),
+        'app/chrome/addons/addon2/components/foo.xpt': foo2_xpt,
+        'app/chrome/addons/addon2/components/bar.xpt': bar_xpt,
     },
 }
 
@@ -94,13 +97,12 @@ RESULT_FLAT = {
     'chrome/f/oo/qux': FILES['chrome/f/oo/qux'],
     'components/components.manifest': [
         'binary-component foo.so',
-        'interfaces interfaces.xpt',
+        'interfaces bar.xpt',
+        'interfaces foo.xpt',
     ],
     'components/foo.so': FILES['components/foo.so'],
-    'components/interfaces.xpt': {
-        'foo': read_interfaces(foo_xpt.open())['foo'],
-        'bar': read_interfaces(bar_xpt.open())['bar'],
-    },
+    'components/foo.xpt': foo_xpt,
+    'components/bar.xpt': bar_xpt,
     'foo': FILES['foo'],
     'app/chrome.manifest': [
         'manifest chrome/chrome.manifest',
@@ -116,7 +118,7 @@ RESULT_FLAT = {
     'app/components/foo.js': FILES['app/components/foo.js'],
 }
 
-for addon in ('addon0', 'addon1'):
+for addon in ('addon0', 'addon1', 'app/chrome/addons/addon2'):
     RESULT_FLAT.update({
         mozpath.join(addon, p): f
         for p, f in {
@@ -125,16 +127,15 @@ for addon in ('addon0', 'addon1'):
                 'manifest components/components.manifest',
             ],
             'chrome/chrome.manifest': [
-                'content content foo/bar/',
+                'content %s foo/bar/' % mozpath.basename(addon),
             ],
             'chrome/foo/bar/baz': FILES[mozpath.join(addon, 'chrome/foo/bar/baz')],
             'components/components.manifest': [
-                'interfaces interfaces.xpt',
+                'interfaces bar.xpt',
+                'interfaces foo.xpt',
             ],
-            'components/interfaces.xpt': {
-                'foo': read_interfaces(foo2_xpt.open())['foo'],
-                'bar': read_interfaces(bar_xpt.open())['bar'],
-            },
+            'components/bar.xpt': bar_xpt,
+            'components/foo.xpt': foo2_xpt,
         }.iteritems()
     })
 
@@ -145,14 +146,16 @@ RESULT_JAR = {
         'chrome/chrome.manifest',
         'components/components.manifest',
         'components/foo.so',
-        'components/interfaces.xpt',
+        'components/foo.xpt',
+        'components/bar.xpt',
         'foo',
         'app/chrome.manifest',
         'app/components/components.manifest',
         'app/components/foo.js',
         'addon0/chrome.manifest',
         'addon0/components/components.manifest',
-        'addon0/components/interfaces.xpt',
+        'addon0/components/foo.xpt',
+        'addon0/components/bar.xpt',
     )
 }
 
@@ -174,7 +177,7 @@ RESULT_JAR.update({
         'foo': FILES['app/chrome/foo/foo'],
     },
     'addon0/chrome/chrome.manifest': [
-        'content content jar:foo.jar!/bar/',
+        'content addon0 jar:foo.jar!/bar/',
     ],
     'addon0/chrome/foo.jar': {
         'bar/baz': FILES['addon0/chrome/foo/bar/baz'],
@@ -184,6 +187,11 @@ RESULT_JAR.update({
         for p, f in RESULT_FLAT.iteritems()
         if p.startswith('addon1/')
     },
+    'app/chrome/addons/addon2.xpi': {
+        mozpath.relpath(p, 'app/chrome/addons/addon2'): f
+        for p, f in RESULT_FLAT.iteritems()
+        if p.startswith('app/chrome/addons/addon2/')
+    },
 })
 
 RESULT_OMNIJAR = {
@@ -203,7 +211,8 @@ RESULT_OMNIJAR.update({
 RESULT_OMNIJAR.update({
     'omni.foo': {
         'components/components.manifest': [
-            'interfaces interfaces.xpt',
+            'interfaces bar.xpt',
+            'interfaces foo.xpt',
         ],
     },
     'chrome.manifest': [
@@ -214,13 +223,17 @@ RESULT_OMNIJAR.update({
     ],
     'app/omni.foo': {
         p: RESULT_FLAT['app/' + p]
-        for p in (
+        for p in chain((
             'chrome.manifest',
             'chrome/chrome.manifest',
             'chrome/foo/foo',
             'components/components.manifest',
             'components/foo.js',
-        )
+        ), (
+            mozpath.relpath(p, 'app')
+            for p in RESULT_FLAT.iterkeys()
+            if p.startswith('app/chrome/addons/addon2/')
+        ))
     },
     'app/chrome.manifest': [],
 })
@@ -234,10 +247,16 @@ RESULT_OMNIJAR['omni.foo'].update({
         'chrome/f/oo/bar/baz',
         'chrome/f/oo/baz',
         'chrome/f/oo/qux',
-        'components/interfaces.xpt',
+        'components/foo.xpt',
+        'components/bar.xpt',
     )
 })
 
+RESULT_OMNIJAR_WITH_SUBPATH = {
+    k.replace('omni.foo', 'bar/omni.foo'): v
+    for k, v in RESULT_OMNIJAR.items()
+}
+
 CONTENTS_WITH_BASE = {
     'bases': {
         mozpath.join('base/root', b) if b else 'base/root': a
@@ -259,6 +278,7 @@ EXTRA_CONTENTS = {
 
 CONTENTS_WITH_BASE['files'].update(EXTRA_CONTENTS)
 
+
 def result_with_base(results):
     result = {
         mozpath.join('base/root', p): v
@@ -267,24 +287,20 @@ def result_with_base(results):
     result.update(EXTRA_CONTENTS)
     return result
 
+
 RESULT_FLAT_WITH_BASE = result_with_base(RESULT_FLAT)
 RESULT_JAR_WITH_BASE = result_with_base(RESULT_JAR)
 RESULT_OMNIJAR_WITH_BASE = result_with_base(RESULT_OMNIJAR)
 
 
-class MockDest(MockDest):
-    def exists(self):
-        return False
-
-
 def fill_formatter(formatter, contents):
-    for base, is_addon in contents['bases'].items():
+    for base, is_addon in sorted(contents['bases'].items()):
         formatter.add_base(base, is_addon)
 
     for manifest in contents['manifests']:
         formatter.add_manifest(manifest)
 
-    for k, v in contents['files'].iteritems():
+    for k, v in sorted(contents['files'].iteritems()):
         if k.endswith('.xpt'):
             formatter.add_interfaces(k, v)
         else:
@@ -294,11 +310,7 @@ def fill_formatter(formatter, contents):
 def get_contents(registry, read_all=False):
     result = {}
     for k, v in registry:
-        if k.endswith('.xpt'):
-            tmpfile = MockDest()
-            registry[k].copy(tmpfile)
-            result[k] = read_interfaces(tmpfile)
-        elif isinstance(v, FileRegistry):
+        if isinstance(v, FileRegistry):
             result[k] = get_contents(v)
         elif isinstance(v, ManifestFile) or read_all:
             result[k] = v.open().read().splitlines()
@@ -313,8 +325,8 @@ class TestFormatters(TestErrors, unittest.TestCase):
     def test_bases(self):
         formatter = FlatFormatter(FileRegistry())
         formatter.add_base('')
-        formatter.add_base('browser')
         formatter.add_base('addon0', addon=True)
+        formatter.add_base('browser')
         self.assertEqual(formatter._get_base('platform.ini'),
                          ('', 'platform.ini'))
         self.assertEqual(formatter._get_base('browser/application.ini'),
@@ -376,6 +388,14 @@ class TestFormatters(TestErrors, unittest.TestCase):
         self.assertEqual(get_contents(registry), RESULT_OMNIJAR_WITH_BASE)
         self.do_test_contents(formatter, CONTENTS_WITH_BASE)
 
+    def test_omnijar_formatter_with_subpath(self):
+        registry = FileRegistry()
+        formatter = OmniJarFormatter(registry, 'bar/omni.foo')
+
+        fill_formatter(formatter, CONTENTS)
+        self.assertEqual(get_contents(registry), RESULT_OMNIJAR_WITH_SUBPATH)
+        self.do_test_contents(formatter, CONTENTS)
+
     def test_omnijar_is_resource(self):
         def is_resource(base, path):
             registry = FileRegistry()
@@ -443,8 +463,8 @@ class TestFormatters(TestErrors, unittest.TestCase):
             f.add_manifest(ManifestContent('chrome', 'foo', 'foo/'))
 
         self.assertEqual(e.exception.message,
-            'Error: "content foo foo/" overrides '
-            '"content foo foo/unix"')
+                         'Error: "content foo foo/" overrides '
+                         '"content foo foo/unix"')
 
         # Chrome with the same name and same flags overrides the previous
         # registration.
@@ -452,8 +472,8 @@ class TestFormatters(TestErrors, unittest.TestCase):
             f.add_manifest(ManifestContent('chrome', 'foo', 'foo/', 'os=WINNT'))
 
         self.assertEqual(e.exception.message,
-            'Error: "content foo foo/ os=WINNT" overrides '
-            '"content foo foo/win os=WINNT"')
+                         'Error: "content foo foo/ os=WINNT" overrides '
+                         '"content foo foo/win os=WINNT"')
 
         # We may start with the more specific entry first
         f.add_manifest(ManifestContent('chrome', 'bar', 'bar/win', 'os=WINNT'))
@@ -462,8 +482,8 @@ class TestFormatters(TestErrors, unittest.TestCase):
             f.add_manifest(ManifestContent('chrome', 'bar', 'bar/unix'))
 
         self.assertEqual(e.exception.message,
-            'Error: "content bar bar/unix" overrides '
-            '"content bar bar/win os=WINNT"')
+                         'Error: "content bar bar/unix" overrides '
+                         '"content bar bar/win os=WINNT"')
 
         # Adding something more specific still works.
         f.add_manifest(ManifestContent('chrome', 'bar', 'bar/win',
@@ -476,9 +496,9 @@ class TestFormatters(TestErrors, unittest.TestCase):
                                     'foo/skin/modern/'))
 
         f.add_manifest(ManifestLocale('chrome', 'foo', 'en-US',
-                                    'foo/locale/en-US/'))
+                                      'foo/locale/en-US/'))
         f.add_manifest(ManifestLocale('chrome', 'foo', 'ja-JP',
-                                    'foo/locale/ja-JP/'))
+                                      'foo/locale/ja-JP/'))
 
         # But same-skin/locale still error out.
         with self.assertRaises(ErrorMessage) as e:
@@ -486,16 +506,16 @@ class TestFormatters(TestErrors, unittest.TestCase):
                                         'foo/skin/classic/foo'))
 
         self.assertEqual(e.exception.message,
-            'Error: "skin foo classic/1.0 foo/skin/classic/foo" overrides '
-            '"skin foo classic/1.0 foo/skin/classic/"')
+                         'Error: "skin foo classic/1.0 foo/skin/classic/foo" overrides '
+                         '"skin foo classic/1.0 foo/skin/classic/"')
 
         with self.assertRaises(ErrorMessage) as e:
             f.add_manifest(ManifestLocale('chrome', 'foo', 'en-US',
-                                         'foo/locale/en-US/foo'))
+                                          'foo/locale/en-US/foo'))
 
         self.assertEqual(e.exception.message,
-            'Error: "locale foo en-US foo/locale/en-US/foo" overrides '
-            '"locale foo en-US foo/locale/en-US/"')
+                         'Error: "locale foo en-US foo/locale/en-US/foo" overrides '
+                         '"locale foo en-US foo/locale/en-US/"')
 
         # Duplicating existing manifest entries is not an error.
         f.add_manifest(ManifestContent('chrome', 'foo', 'foo/unix'))
