@@ -4,27 +4,25 @@
 
 from __future__ import absolute_import
 
-import sys
 from setuptools import setup, find_packages
 
 PACKAGE_NAME = 'mozrunner'
-PACKAGE_VERSION = '6.14'
+PACKAGE_VERSION = '7.4.0'
 
 desc = """Reliable start/stop/configuration of Mozilla Applications (Firefox, Thunderbird, etc.)"""
 
-deps = ['mozdevice >= 0.37',
-        'mozfile >= 1.0',
-        'mozinfo >= 0.7',
-        'mozlog >= 3.0',
-        'mozprocess >= 0.23',
-        'mozprofile >= 0.18',
-        ]
+deps = [
+    'mozdevice>=3.0.1',
+    'mozfile>=1.2',
+    'mozinfo>=0.7,<2',
+    'mozlog~=4.1',
+    'mozprocess>=0.23,<2',
+    'mozprofile~=2.1',
+    'six>=1.10.0,<2',
+]
 
 EXTRAS_REQUIRE = {'crash': ['mozcrash >= 1.0']}
 
-# we only support python 2 right now
-assert sys.version_info[0] == 2
-
 setup(name=PACKAGE_NAME,
       version=PACKAGE_VERSION,
       description=desc,
@@ -34,7 +32,8 @@ setup(name=PACKAGE_NAME,
                    'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
                    'Natural Language :: English',
                    'Operating System :: OS Independent',
-                   'Programming Language :: Python',
+                   'Programming Language :: Python :: 2.7',
+                   'Programming Language :: Python :: 3.5',
                    'Topic :: Software Development :: Libraries :: Python Modules',
                    ],
       keywords='mozilla',
